// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/

#define DEBUG_BC

#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include "codebloc.h"

struct instr
{
  int bytect;
  int o_ct;
  int doshift;
  BYTE mask;
  BYTE b1;
  BYTE b2;
  BYTE b3;
  char *descr;
};

struct instr instrTable[] = {


/*BC_POP */                  1,1,0, 0xFF,0x00,0x00,0x00,"BC_POP",
/*BC_DUP */                  1,1,0, 0xFF,0x01,0x00,0x00,"BC_DUP",
/*BC_RETURN */               1,1,0, 0xFF,0x02,0x00,0x00,"BC_RETURN",
/*BC_SELF */                 1,1,0, 0xFF,0x03,0x00,0x00,"BC_SELF",
/*BC_CLOSURE */              1,1,0, 0xFF,0x04,0x00,0x00,"BC_CLOSURE",
/*BC_FOREACHNEXT */          1,1,0, 0xFF,0x05,0x00,0x00,"BC_FOREACHNEXT",
/*BC_FOREACHCOMPLETE */      1,1,0, 0xFF,0x06,0x00,0x00,"BC_FOREACHCOMPLETE",
/*BC_ENDEXCEPTION */         3,3,0, 0xFF,0x07,0x00,0x07,"BC_ENDEXCEPTION",
/*BC_PUSHLITERAL */          1,1,0, 0xF8,0x18,0x00,0x00,"BC_PUSHLITERAL",
/*BC_PUSHIMMEDIATE */        1,1,1, 0xF8,0x20,0x00,0x00,"BC_PUSHIMMEDIATE",
/*BC_CALLGLOBAL */           1,1,0, 0xF8,0x28,0x00,0x00,"BC_CALLGLOBAL",
/*BC_CALLWITH */             1,1,0, 0xF8,0x30,0x00,0x00,"BC_CALLWITH",
/*BC_SENDMESSAGE */          1,1,0, 0xF8,0x38,0x00,0x00,"BC_SENDMESSAGE",
/*BC_CONDSENDMESSAGE */      1,1,0, 0xF8,0x40,0x00,0x00,"BC_CONDSENDMESSAGE",
/*BC_INHSENDMESSAGE */       1,1,0, 0xF8,0x48,0x00,0x00,"BC_INHSENDMESSAGE",
/*BC_CONDINHSENDMESSAGE */   1,1,0, 0xF8,0x50,0x00,0x00,"BC_CONDINHSENDMESSAGE",
/*BC_GOTO */                 1,3,0, 0xF8,0x58,0x00,0x00,"BC_GOTO",
/*BC_GOTOIFNIL */            1,3,0, 0xF8,0x60,0x00,0x00,"BC_GOTOIFNIL",
/*BC_GOTOIFNOTNIL */         1,3,0, 0xF8,0x68,0x00,0x00,"BC_GOTOIFNOTNIL",
/*BC_PUSHEXTERNAL */         1,1,0, 0xF8,0x70,0x00,0x00,"BC_PUSHEXTERNAL",
/*BC_PUSHLOCAL */            1,1,0, 0xF8,0x78,0x00,0x00,"BC_PUSHLOCAL",
/*BC_CREATEFRAME */          1,1,0, 0xF8,0x80,0x00,0x00,"BC_CREATEFRAME",
/*BC_CREATEARRAY */          1,1,0, 0xF8,0x88,0x00,0x00,"BC_CREATEARRAY",
/*BC_PUSHSLOTVALUE */        1,1,0, 0xFF,0x91,0x00,0x00,"BC_PUSHSLOTVALUE",
/*BC_ASSIGNTOSLOT */         1,1,0, 0xFF,0x98,0x00,0x00,"BC_ASSIGNTOSLOT",
/*BC_ASSIGNTOSLOTANDPUSH */  1,1,0, 0xFF,0x99,0x00,0x00,"BC_ASSIGNTOSLOTANDPUSH",
/*BC_ASSIGNTOLOCAL */        1,1,0, 0xF8,0xA0,0x00,0x00,"BC_ASSIGNTOLOCAL",
/*BC_ASSIGNTOEXTERNAL */     1,1,0, 0xF8,0xA8,0x00,0x00,"BC_ASSIGNTOEXTERNAL",
/*BC_INCREMENTLOCAL */       1,1,0, 0xF8,0xB0,0x00,0x00,"BC_INCREMENTLOCAL",
/*BC_FORLOOPGOTO */          1,3,0, 0xF8,0xB8,0x00,0x00,"BC_FORLOOPGOTO",
/*BC_ADD */                  1,1,0, 0xFF,0xC0,0x00,0x00,"BC_ADD",
/*BC_SUB */                  1,1,0, 0xFF,0xC1,0x00,0x00,"BC_SUB",
/*BC_DEREF */                1,1,0, 0xFF,0xC2,0x00,0x00,"BC_DEREF",
/*BC_ASSIGNTOARRAYANDPUSH */ 1,1,0, 0xFF,0xC3,0x00,0x00,"BC_ASSIGNTOARRAYANDPUSH",
/*BC_COMPEQ */               1,1,0, 0xFF,0xC4,0x00,0x00,"BC_COMPEQ",
/*BC_NOT */                  1,1,0, 0xFF,0xC5,0x00,0x00,"BC_NOT",
/*BC_COMPNEQ */              1,1,0, 0xFF,0xC6,0x00,0x00,"BC_COMPNEQ",
/*BC_MULT */                 3,3,0, 0xFF,0xC7,0x00,0x07,"BC_MULT",
/*BC_FDIV */                 3,3,0, 0xFF,0xC7,0x00,0x08,"BC_FDIV",
/*BC_IDIV */                 3,3,0, 0xFF,0xC7,0x00,0x09,"BC_IDIV",
/*BC_COMPLT */               3,3,0, 0xFF,0xC7,0x00,0x0A,"BC_COMPLT",
/*BC_COMPGT */               3,3,0, 0xFF,0xC7,0x00,0x0B,"BC_COMPGT",
/*BC_COMPGE */               3,3,0, 0xFF,0xC7,0x00,0x0C,"BC_COMPGE",
/*BC_COMPLE */               3,3,0, 0xFF,0xC7,0x00,0x0D,"BC_COMPLE",
/*BC_BAND */                 3,3,0, 0xFF,0xC7,0x00,0x0E,"BC_BAND",
/*BC_BOR */                  3,3,0, 0xFF,0xC7,0x00,0x0F,"BC_BOR",
/*BC_BNOT */                 3,3,0, 0xFF,0xC7,0x00,0x10,"BC_BNOT",
/*BC_FOREACH */              3,3,0, 0xFF,0xC7,0x00,0x11,"BC_FOREACH",
/*BC_LENGTH */               3,3,0, 0xFF,0xC7,0x00,0x12,"BC_LENGTH",
/*BC_CLONE */                3,3,0, 0xFF,0xC7,0x00,0x13,"BC_CLONE",
/*BC_SETCLASS */             3,3,0, 0xFF,0xC7,0x00,0x14,"BC_SETCLASS",
/*BC_ADDARRAYSLOT */         3,3,0, 0xFF,0xC7,0x00,0x15,"BC_ADDARRAYSLOT",
/*BC_MAKESTRING */           3,3,0, 0xFF,0xC7,0x00,0x16,"BC_MAKESTRING",
/*BC_SLOTEXISTS */           3,3,0, 0xFF,0xC7,0x00,0x17,"BC_SLOTEXISTS",
/*BC_CLASSOF */              3,3,0, 0xFF,0xC7,0x00,0x18,"BC_CLASSOF",
/*BC_ONEXCEPTION */          1,1,0, 0xF8,0xC8,0x00,0x00,"BC_ONEXCEPTION",

};


char *Instruction::Descr()
{
  return instrTable[inumber].descr;
}

Instruction::dumpinst(ofstream *outFile)
{
  int i;

  *outFile << "[";
  for (i=0;i<instLen;++i)
    outFile->form("0x%2.2x,",result[i]);
  *outFile << "]";
}

Instruction::dump(ofstream *outFile)
{
#ifdef DEBUG_BC
  *outFile <<offset <<":" << Descr() <<" parmtype " << parmtype << " parm " << iparam <<" len " << instLen << " Instr " << inumber;
  dumpinst(outFile);
  *outFile << endl;
#else
  for (int i=0;i<instLen;++i)
    outFile->form("0x%2.2x,",result[i]);
#endif

}

String Instruction::FinalCode()
{
  String t;
  
  t = "";
  
  for (int i=0;i<instLen;++i)
    t = t + result[i];
  return t;
}


Instruction::Reduce()
{
  int rc;
  struct instr *xx;

  xx = &instrTable[inumber];
  rc = bytepack(result,xx->b1,xx->b2,xx->b3,xx->bytect,xx->o_ct,iparam,parmtype,xx->doshift);
  instLen = rc;
}


int Instruction::bytepack(BYTE *aa,BYTE b1,BYTE b2,BYTE b3,BYTE bc,int o_ct,int parm,enum iparmtype t,int doshift)
{
  BYTE tt;
  BYTE hb;
  BYTE lb;
  int parmval;
  
  if (bc > 1) 
    {
      aa[0] = b1;
      aa[1] = b2;
      aa[2] = b3;
      return 3;
    }
  
  switch (t)
    {
    case simple:
      if (doshift)
	parmval = parm << 2;
      else
	parmval = parm;
      break;
    case ptcomplex:
      if (doshift)
	parmval = (parm << 2) + 1;
      else
	parmval = parm;
      break;
    case unique:
      parmval = parm; 
      break;
    case magic:
      if (doshift)
	parmval = (parm << 2) + 3;
      else
	parmval = parm;
      break;
    default:
      if (doshift)
	parmval = parm << 2;
      else
	parmval = parm;
      break;
    }
	

  
  if (parmval > 6)
    {
      aa[0] =  b1 + 0x07;
      hb = (BYTE)((parmval>>8) & 0xFF00);
      aa[1] = hb;
      lb = parmval & 0xFF;
      aa[2] = lb;
      return 3;
    }
  else
    {
      tt = (BYTE)parmval;
      *aa = b1 + tt;
      if (o_ct > 1)
	{
	  aa[1] = 0;
	  aa[2] = 0;
	  return o_ct;
	}
      else
	return 1;
    }      
}



Instruction::Backpatch()
{
  BYTE tt;
  BYTE hb;
  BYTE lb;
  
  if (doBackpatch)
    {
      if (patchTo)
	{
	  if (afterPatch)
	    iparam = patchTo->Offset() + patchTo->instLen;
	  else
	    iparam = patchTo->Offset();
	  
	  
	  result[0] =  result[0] + 0x07;
	  hb = (BYTE)((iparam>>8) & 0xFF00);
	  result[1] = hb;
	  lb = iparam & 0xFF;
	  result[2] = lb;
	  instLen = 3;
	}
      else
	{
	  iparam = 0;
	  result[0] =  result[0] + 0x07;
	  hb = (BYTE)((iparam>>8) & 0xFF00);
	  result[1] = hb;
	  lb = iparam & 0xFF;
	  result[2] = lb;
	  instLen = 3;
	}
    }
}


Instruction *Codeblock::AppendInstruction(int n,int p,enum iparmtype tp)
{
  Instruction *t = new Instruction(n,p,tp);
  
  theInstructions.append(t);
  return t;
}


Codeblock::dump(ofstream *outFile)
{
#ifdef DEBUG_BC
  *outFile << "------dumping bytecodes ----------"<<endl;
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->dump(outFile);
    };    
  *outFile << "------finished dumping bytecodes ----------"<<endl;
#else
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->dump(outFile);
    };    
#endif
}




Codeblock::Reduce()
{
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->Reduce();
    };    
}

Codeblock::ComputeOffsets()
{
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  int offset = 0;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->SetOffset(offset);
    };    
  codeSize = offset;
}

Codeblock::Backpatch()
{
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  int offset = 0;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->Backpatch();
    };    
}

Codeblock::FinalCode()
{
  InstructionList tempList = theInstructions;;
  Instruction *tempItem;
  int offset = 0;
  
  theCode = "";

  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      theCode += tempItem->FinalCode();
    };    
}



#define	BC_ARG_MASK	0x7		/* byte code argument */
#define	BC_ARG_3_BYTES	0x7		/* use next 2 also */
#define	BC_3_BYTES(_bc)	(((_bc) & BC_ARG_MASK) == BC_ARG_3_BYTES)

#define NUMELEMENTS(v) (sizeof (v) / sizeof (v[0]))


struct instr *find_bc(BYTE *ip)
{
  int   elements = NUMELEMENTS(instrTable);
  int i;
  BYTE fb;
  BYTE tb;

  for (i=0;i<elements;++i)
    {
      fb = ip[0] & instrTable[i].mask;
      if (instrTable[i].b1 == fb)
	{
	  if (instrTable[i].bytect == 1)
	    return &instrTable[i];
	  else /* three byte instruction */
	    {
	      if (instrTable[i].b2 == ip[1])
		{
		  if (instrTable[i].b3 == ip[2])
		    {
		      return &instrTable[i];
		    }
		}
	    }
	}
    }
  return NULL;
}





int show_instructions(BYTE *ip, int cnt) {
	BYTE	*ep = ip + cnt;
	struct instr *inst;
	BYTE br;
    
	while (ip < ep) {
	  inst = find_bc(ip);
	  if (inst)
	    {

	      if (BC_3_BYTES(*ip)) {
		printf("  0x%02x %02x %02x %s",ip[0],ip[1],ip[2],inst->descr);
		ip += 3;
	      } else {
		br = (*ip) & 0x07;
		printf(" %02x         %s 0x%02x", ip[0],inst->descr,br);
		ip++;
	      }
	      printf("\n");
	    }
	  else
	    {
	      printf(" data, bc 0x%02x", *ip);
	      if (BC_3_BYTES(*ip)) {
		printf(" 0x%02x%02x", ip[1], ip[2]);
		ip += 3;
	      } else {
		ip++;
	      }
	      printf("\n");
	    }
	}
}
