// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#ifndef CODEBLOCK_H_INC
#define CODEBLOCK_H_INC

#include <iostream.h>
#include <fstream.h>
#include <DLList.h>
#include <String.h>

#define BC_POP                   0
#define BC_DUP                   1
#define BC_RETURN                2
#define BC_SELF                  3
#define BC_CLOSURE               4
#define BC_FOREACHNEXT           5
#define BC_FOREACHCOMPLETE       6
#define BC_ENDEXCEPTION          7
#define BC_PUSHLITERAL           8
#define BC_PUSHIMMEDIATE         9
#define BC_CALLGLOBAL            10
#define BC_CALLWITH              11
#define BC_SENDMESSAGE           12
#define BC_CONDSENDMESSAGE       13
#define BC_INHSENDMESSAGE        14
#define BC_CONDINHSENDMESSAGE    15
#define BC_GOTO                  16
#define BC_GOTOIFNIL             17
#define BC_GOTOIFNOTNIL          18
#define BC_PUSHEXTERNAL          19
#define BC_PUSHLOCAL             20
#define BC_CREATEFRAME           21
#define BC_CREATEARRAY           22
#define BC_PUSHSLOTVALUE         23
#define BC_ASSIGNTOSLOT          24
#define BC_ASSIGNTOSLOTANDPUSH   25
#define BC_ASSIGNTOLOCAL         26
#define BC_ASSIGNTOEXTERNAL      27
#define BC_INCREMENTLOCAL        28
#define BC_FORLOOPGOTO           29
#define BC_ADD                   30
#define BC_SUB                   31
#define BC_DEREF                 32
#define BC_ASSIGNTOARRAYANDPUSH  33
#define BC_COMPEQ                34
#define BC_NOT                   35
#define BC_COMPNEQ               36
#define BC_MULT                  37
#define BC_FDIV                  38
#define BC_IDIV                  39
#define BC_COMPLT                40
#define BC_COMPGT                41
#define BC_COMPGE                42
#define BC_COMPLE                43
#define BC_BAND                  44
#define BC_BOR                   45
#define BC_BNOT                  46
#define BC_FOREACH               47
#define BC_LENGTH                48
#define BC_CLONE                 49
#define BC_SETCLASS              50
#define BC_ADDARRAYSLOT          51
#define BC_MAKESTRING            52
#define BC_SLOTEXISTS            53
#define BC_CLASSOF               54
#define BC_ONEXCEPTION           55




#define BYTE unsigned char

#define VAL_NIL 0x02
#define VAL_TRUE 0x1a

enum iparmtype{none,simple,ptcomplex,unique,magic};

class Instruction
{
  int inumber;
  int iparam;
  enum iparmtype parmtype;
  BYTE result[8];
  int instLen;
  int offset;
  Instruction *patchTo;
  int doBackpatch;
  int afterPatch;
 public:
  Instruction(int n,int p,enum iparmtype t):inumber(n),iparam(p),parmtype(t) {doBackpatch = 0;}
  char *Instruction::Descr();
  dump(ofstream *outFile);
  //does the final encoding of the instruction after backpatching
  String FinalCode();
  Instruction::dumpinst(ofstream *outFile);
  //reduces instruction lengths if possible, and resets the instLen
  Reduce();

  SetPatchTo(Instruction *p) {patchTo = p;afterPatch = 0;doBackpatch = 1;}

  SetPatchAfter(Instruction *p) {patchTo = p;afterPatch = 1;doBackpatch = 1;}

  //backpatches an instruction
  Backpatch();
  
  Offset() {return offset;}
  SetOffset(int &ofs) {offset = ofs;ofs += instLen;}
  int bytepack(BYTE *aa,BYTE b1,BYTE b2,BYTE b3,BYTE bc,int o_ct,int parm,enum iparmtype t,int doshift);


};


typedef DLList<Instruction *> InstructionList;

class Codeblock
{
  InstructionList theInstructions;
  int codeSize;
  String theCode;
 public:

  Instruction *AppendInstruction(int n,int p,enum iparmtype tp);
  Instruction *LastInstr() {if (theInstructions.empty) return NULL; else return theInstructions.rear();}
  Instruction *FirstInstr() {if (theInstructions.empty) return NULL; else return theInstructions.front();}
  dump(ofstream *outFile);
  Reduce();
  ComputeOffsets();
  Backpatch();
  FinalCode();  
  String getFinalCode()
    {return theCode;}
  
};


#endif
