// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#include "objects.h"
#include "newtobjs.h"


int stackdepth = 0;
extern ObjectFile *theObjectFile;

extern Symtab globalSymbolTable;


NO_Symbol *GetSym(String &nv)
{
  NO_Symbol *tt;
  tt = globalSymbolTable.getNOSymbol(nv);
  return tt;
}

NO_Symbol *GetSym(char *sp)
{
  NO_Symbol *tt;
  String ts = String(sp);
  tt = globalSymbolTable.getNOSymbol(ts);
  return tt;
}

NO_Symbol *GetLocalSym(String &nv)
{
  NO_Symbol *tt;
  tt = globalSymbolTable.getNOSymbol(nv);
  return tt;
}

NO_Symbol *GetLocalSym(char *sp)
{
  NO_Symbol *tt;
  String ts = String(sp);
  tt = globalSymbolTable.getNOSymbol(ts);
  return tt;
}

#define VPUMNIL 0

NO_VpumUnusual *GetNilSym()
{
  NO_VpumUnusual *tt;
  tt = new NO_VpumUnusual(VPUMNIL);
  return tt;
}



MakePkgHeader()
{
  SymtabEntry *authCpy;
  SymtabEntry *pkgName;
  SymtabEntry *partTp;
  SymtabEntry *partStr;

  authCpy = globalSymbolTable.FindEntry("authorCopyright");
  pkgName = globalSymbolTable.FindEntry("packageName");
  partTp = globalSymbolTable.FindEntry("partType");
  partStr = globalSymbolTable.FindEntry("partString");

  if (authCpy)
    authCpy->dump();
  if (pkgName)
    pkgName->dump();
  if (partTp)
    partTp->dump();
  if (partStr)
    partStr->dump();

}



virtual int Input::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  ConstituentList tempList = theConstituents;
  Constituent *tempItem;
  MakePkgHeader();
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}

virtual int ExpressionSequence::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  int didit = 0;
  int onstack = 0;

  ExpressionList tempExpressions = theExpressions;
  Expression *tempExpression;

  //loop over all of the expressions
  while (!tempExpressions.empty())
    {
      //if there is something left on the stack, pop it
      if (onstack)
	{
	  theCode->AppendInstruction(BC_POP,0,none);
	  onstack = 0;
	}
      tempExpression = tempExpressions.remove_front();
      //emit the expression
      onstack = tempExpression->emit(theCode,theContainer);
    };
  return onstack;
}

virtual int Symbol::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //push the symbol value on the stack
  symbolTableRef->emit(theCode,theContainer);
  return 1;
}

virtual int Symbol::emitAssignTo(Codeblock *theCode)
{
  //save stack value to the symbol
  symbolTableRef->emitAssignTo(theCode);
  return 0;
}
virtual int Symbol::emitIncr(Codeblock *theCode)
{
  //increment the symbol value
  symbolTableRef->emitIncr(theCode);
  return 1;
}

virtual int FormalArgumentList::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO is there anything to do here?
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}


virtual int FormalArgumentList::AddArgsToFrame(NO_Frame *theFrame)
{
  //add the arguments to the frame, and make their value nil.
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();

      theFrame->AddSlot(tempItem->GetNOSymbol(),GetNilSym());
    };
  return 0;
}



virtual int InitClause::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO check if this is a local (probably not needed, cause globals not allowed)
  if (theExpression != NULL)
    {
      theExpression->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_ASSIGNTOLOCAL,theSymbol->Order()+2,simple);
    }
  --stackdepth;
  return -1;
}



virtual int GlobalFunctionDecl::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO we need to generate global function declarations correctly
  theObjectFile->outFile << "GlobalFunctionDecl ";
  theObjectFile->outFile << " IsGlobal " << isGlobal;
  theSymbol->emit(theCode,theContainer);
  theArgs->emit(theCode,theContainer);
  theExpression->emit(theCode,theContainer);
  return 0;
}


virtual int GlobalWithInitialization::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile  << "GlobalWithInitialization";
  theInitClause->emit(theCode,theContainer);
  return 0;
}

//TODO this must be filled in
callGlobalFunction(Codeblock *theCode,NO_Frame *theContainer)
{
}

virtual int SimpleExpression::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  switch(theOperator)
    {
    case 0: 
      theCode->AppendInstruction(BC_SELF,0,none);
      break;
    case 1:   
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_ADD,0,none);
      break;
    case 2:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_SUB,0,none);
      break;
    case 3:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_MULT,0,none);
      break;
    case 4:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_FDIV,0,none);
      break;
    case 5:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_IDIV,0,none);
      break;
    case 6:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      //TODO  call global func mod
      break;
    case 7:	
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      //TODO call global ls
      break;
    case 8:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      //TODO call global rs
      break;
    case 9:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPEQ,0,none);
      break;
    case 10:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPNEQ,0,none);
      break;
    case 11:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPLT,0,none);
      break;
    case 12:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPGT,0,none);
      break;
    case 13:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPLE,0,none);
      break;
    case 14:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_COMPGE,0,none);
      break;
    case 15:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_BAND,0,none);
      break;
    case 16:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_BOR,0,none);
      break;
    case 17:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      break;
    case 18:
      left_expr->emit(theCode,theContainer);
      right_expr->emit(theCode,theContainer);
      break;
    case 19:
      left_expr->emit(theCode,theContainer);
      break;
    case 20:
      left_expr->emit(theCode,theContainer);
      theCode->AppendInstruction(BC_NOT,0,none);
      break;
    case 21:
      theCode->AppendInstruction(BC_SELF,0,none);
      break;
    }
  return 1;
}


virtual int CompoundExpression::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  int rc;
  if (theExpressions)
     rc = theExpressions->emit(theCode,theContainer);
  return rc;
}

virtual int SimpleObject::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "SimpleObject";
  theSimpleLiteral->emit(theCode,theContainer);
  return 0;
}

virtual int PathExpression::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  theObjectFile->outFile << "PathExpression ";
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}


virtual int PathObject::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "PathObjectLiteral";
  theObject->emit(theCode,theContainer);
  return 0;
}


virtual int Array::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "Array ";
  if (initialSymbol != NULL)
    initialSymbol->emit(theCode,theContainer);
  ObjectList tempList = theObjects;
  Object *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}

virtual int ArrayObject::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "ArrayObjectLiteral";
  theObject->emit(theCode,theContainer);
  return 0;
}


virtual int FrameSlot::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "FrameSlot";
  theSymbol->emit(theCode,theContainer);
  theObject->emit(theCode,theContainer);
  return 0;
}

virtual int Frame::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "Frame ";
  FrameSlotList tempList = theSlots;
  FrameSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}

virtual int FrameObject::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "FrameObjectLiteral";
  theObject->emit(theCode,theContainer);
  return 0;
}

virtual int ArrayConstructor::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << endl <<"BEGIN ArrayConstructor ";
  if (initialSymbol != NULL)
    initialSymbol->emit(theCode,theContainer);
  ExpressionList tempList = theExpressions;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  theObjectFile->outFile << endl <<"END ArrayConstructor ";
  return 0;
}


virtual int FrameConstructorSlot::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "FrameConstructorSlot";
  theSymbol->emit(theCode,theContainer);
  theExpression->emit(theCode,theContainer);
  return 0;
}

virtual int FrameConstructor::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << endl << "BEGIN FrameConstructor ";
  FSSlotList tempList = theSlots;
  FrameConstructorSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  theObjectFile->outFile << endl << "END FrameConstructor ";
  return 0;
}


/*------------------------------------------ */

virtual int FunctionConstructor::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Codeblock localCode;
  NO_Frame *fnFrame;
  NO_Frame *argFrame;
  NO_Array *litArray;
  NO_VpumPtr *tempPtr;
  NO_Data *theCodeData;
  String theCode;
  
  fnFrame = new NO_Frame;
  
  litArray = new NO_Array(NOT_None);

  fnFrame->SetFunctionFrame(litArray);

  fnFrame->AddSlot(GetSym("class"),GetSym("CodeBlock"));

  theExpression->emit(&localCode,fnFrame);
  localCode.AppendInstruction(BC_RETURN,0,none);    

  localCode.Reduce();
  localCode.ComputeOffsets();
  localCode.Backpatch();
  localCode.FinalCode();
  theCode = localCode.getFinalCode();

  theCodeData = new NO_Data(theCode);
  theCodeData->SetCode();
  tempPtr = theCodeData->CreateVPointer();
  //localCode.dump(&theObjectFile->outFile);
  // get codeblock representation, add it to the slot
  fnFrame->AddSlot(GetSym("instructions"),tempPtr);
  

  tempPtr = litArray->CreateVPointer();
  fnFrame->AddSlot(GetSym("literals"),tempPtr);

  argFrame = new NO_Frame;
  tempPtr = argFrame->CreateVPointer();
  argFrame->AddSlot(GetSym("_nextArgFrame"),GetNilSym());
  argFrame->AddSlot(GetSym("_Parent"),GetNilSym());
  argFrame->AddSlot(GetSym("_implementor"),GetNilSym());

  fnFrame->AddSlot(GetSym("argFrame"),argFrame);

  int argCnt  = 0;

  if (theArgs)
    {
      theArgs->AddArgsToFrame(argFrame);
    argCnt = theArgs->countArgs();
    }
  theSymbolTable.AddLocalsToFrame(argFrame);

  NO_VpumInt *nint = new NO_VpumInt(argCnt);
  fnFrame->AddSlot(GetSym("numArgs"),nint);
  return 0;
}


virtual int FrameAccessor::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "FrameAccessor";
  leadingExpr->emit(theCode,theContainer);
  if (theSymbol != NULL)
    theSymbol->emit(theCode,theContainer);
  if (trailingExpr != NULL)
    trailingExpr->emit(theCode,theContainer);
  return 0;
}
virtual int FrameAccessor::emitAssignTo(Codeblock *theCode)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "FrameAccessor";
//  leadingExpr->emit(theCode);
//  if (theSymbol != NULL)
//    theSymbol->emit(theCode,theContainer);
//  if (trailingExpr != NULL)
//    trailingExpr->emit(theCode,theContainer);
  return 0;
}

virtual int ArrayAccessor::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "ArrayAccessor";
  leadingExpr->emit(theCode,theContainer);
  trailingExpr->emit(theCode,theContainer);
  return 0;
}
virtual int ArrayAccessor::emitAssignTo(Codeblock *theCode)
{
  theObjectFile->outFile << "ArrayAccessor";
//  leadingExpr->emit(theCode,theContainer);
//  trailingExpr->emit(theCode,theContainer);
  return 0;
}


virtual int Assignment::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theExpression->emit(theCode,theContainer);
  //TODO we need to get the lvalue type and assign correctly
  theLvalue->emitAssignTo(theCode);
  theLvalue->emit(theCode,theContainer);
  return 0;
}

virtual int SymbolExists::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in
  theObjectFile->outFile << "SymbolExists";
  theSymbol->emit(theCode,theContainer);
  return 0;
}

virtual int FrameAccExists::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in
  theObjectFile->outFile << "FrameAccExists";
  theFrameAcc->emit(theCode,theContainer);
  return 0;
}

virtual int ExprExists::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in
  theObjectFile->outFile << "ExprExists";
  if (theExpression)
    theExpression->emit(theCode,theContainer);
  theSymbol->emit(theCode,theContainer);
  return 0;
}

virtual int ActualArgumentList::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "ActualArgumentList";
  
  ExpressionList tempList = theArgs;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}


virtual int FunctionCall::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this is totally bogus
  theObjectFile->outFile << "FunctionCall";
  if (theSymbol != NULL)
    theSymbol->emit(theCode,theContainer);
  if (theExpression != NULL)
    theExpression->emit(theCode,theContainer);
  if (theArgs != NULL)
    theArgs->emit(theCode,theContainer);
  return 0;
}

virtual int MessageSend::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  int argCnt  = 0;
  Instruction *i1;

  theObjectFile->outFile << "MessageSend";
  theObjectFile->outFile << " IsInherited " << isInherited;
  if (theExpression)
    theExpression->emit(theCode,theContainer);
  theObjectFile->outFile << " ColOrCQ " << col_or_cq;
  if (theArgs)
    {
      theArgs->emit(theCode,theContainer);
      argCnt = theArgs->countArgs();
    }
  if (theSymbol)
    theSymbol->emit(theCode,theContainer);

  if (isInherited)
    {
      if (col_or_cq)
	{
	  i1 = theCode->AppendInstruction(BC_CONDINHSENDMESSAGE,argCnt,simple);  
	}
      else
	{
	  i1 = theCode->AppendInstruction(BC_INHSENDMESSAGE,argCnt,simple);  
	}
    }
  else
    {
      if (col_or_cq)
	{
	  i1 = theCode->AppendInstruction(BC_CONDSENDMESSAGE,argCnt,simple);  
	}
      else
	{
	  i1 = theCode->AppendInstruction(BC_SENDMESSAGE,argCnt,simple);  
	}
    }
}

virtual int IfExpression::emit(Codeblock *theCode,NO_Frame *theContainer)
{  
  Instruction *i1;
  Instruction *i2;
  Instruction *i3;
  int ic;

  if (boolExpr != NULL)
    {
      ic = boolExpr->emit(theCode,theContainer);
      if (ic) 
	theCode->AppendInstruction(BC_POP,0,none);  
    }
  i1 = theCode->AppendInstruction(BC_GOTOIFNOTNIL,0,none);  
  if (thenExpr != NULL)
    {
      ic = thenExpr->emit(theCode,theContainer);
      if (ic) 
	theCode->AppendInstruction(BC_POP,0,none);  
    }
  i2 = theCode->AppendInstruction(BC_GOTO,0,none);  
  i1->SetPatchAfter(i2);
  if (elseExpr != NULL)
    {
      ic = elseExpr->emit(theCode,theContainer);
      if (ic) 
	theCode->AppendInstruction(BC_POP,0,none);  
    }
  i3 = theCode->LastInstr();
  i2->SetPatchAfter(i3);
  return 0;
}

typedef DLList<Iteration *> IterStack;
IterStack enclosingLoop;

virtual int InfiniteLoop::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Instruction *l;
  Instruction *i1;
  int ic;
  Iteration *iter;

  iter = (Iteration *)this;
  enclosingLoop.prepend(iter);
  //get the location of the last instruction
  l = theCode->LastInstr();
  //emit instructions for the expression
  ic = theExpression->emit(theCode,theContainer);
  //if there are items on the stack, pop them off
  if (ic) 
    theCode->AppendInstruction(BC_POP,0,none);  
  //append a goto instruction 
  i1 = theCode->AppendInstruction(BC_GOTO,0,none);  
  //make the goto point to the previous instruction
  i1->SetPatchAfter(l);
  enclosingLoop.del_front();
  //there is nothing on the stack
  return 0;
}

virtual int ForLoop::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Instruction *p1;
  Instruction *p2;
  Instruction *p3;
  int ic;
  Iteration *iter;

  iter = (Iteration *)this;
  enclosingLoop.prepend(iter);

  //emit the initial expression
  initialExpression->emit(theCode,theContainer);
  //save in the loop symbol
  loopSymbol->emitAssignTo(theCode);
  //emit the ending condition
  toExpression->emit(theCode,theContainer);
  //save in the limit symbol;
  limitSymbol->emitAssignTo(theCode);
  //if there is a by expression, emit it, and save in incrSymbol
  if (byExpression != NULL)
    {
      byExpression->emit(theCode,theContainer);
      incrSymbol->emitAssignTo(theCode);
    }
  else // set the incrSymbol to 1
    {      
      theCode->AppendInstruction(BC_PUSHIMMEDIATE,1,simple);  
      incrSymbol->emitAssignTo(theCode);
    }
  //push the increment symbol
  incrSymbol->emit(theCode,theContainer);  
  //push the loop symbol
  loopSymbol->emit(theCode,theContainer);  
  //go to the test
  p1 = theCode->AppendInstruction(BC_GOTO,0,none);  
  //emit the loop expression
  ic = doExpression->emit(theCode,theContainer);
  //if it left anything on the stack, pop it off
  if (ic) 
    theCode->AppendInstruction(BC_POP,0,none);  
  //emit the incrementing symbol
  incrSymbol->emit(theCode,theContainer);
  //push the increment value
  loopSymbol->emitIncr(theCode);
  //push the limit value
  limitSymbol->emit(theCode,theContainer);
  p2 = theCode->LastInstr();
  //make p1 jump here
  p1->SetPatchTo(p2);
  //append the looping test
  p3 = theCode->AppendInstruction(BC_FORLOOPGOTO,0,none);  
  //set patch to after p1
  p3->SetPatchAfter(p1);
  //push nil value
  p3 = theCode->AppendInstruction(BC_PUSHIMMEDIATE,VAL_NIL,unique);  
  PatchBreaksAfter(p3);
  enclosingLoop.del_front();
  //there is one item on the stack
  return 1;
}

virtual int ForEachLoop::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in
  theObjectFile->outFile << "ForEachLoop";
  initialSymbol->emit(theCode,theContainer);
  if (nextSymbol != NULL)
    nextSymbol->emit(theCode,theContainer);
  theObjectFile->outFile << " IsDeeply " <<isDeeply;
  inExpression->emit(theCode,theContainer);
  theObjectFile->outFile << " IsDo " <<isDo;
  doExpression->emit(theCode,theContainer);
  return 1;
}

virtual int WhileLoop::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Instruction *l;
  Instruction *p1;
  Instruction *p2;
  Instruction *p3;
  int ic;
  Iteration *iter;

  iter = (Iteration *)this;
  enclosingLoop.prepend(iter);

  p1 = theCode->AppendInstruction(BC_GOTO,0,none);  
  //emit instructions for the expression
  ic = doExpression->emit(theCode,theContainer);
  //if there are items on the stack, pop them off
  if (ic) 
    theCode->AppendInstruction(BC_POP,0,none);  

  //emit the initial expression
  ic = whileExpression->emit(theCode,theContainer);
  p3 = theCode->LastInstr();
  p1->SetPatchTo(p3);
  p2 = theCode->AppendInstruction(BC_GOTOIFNIL,0,none);  
  p2->SetPatchAfter(p1);
  p3 = theCode->AppendInstruction(BC_PUSHIMMEDIATE,VAL_NIL,unique);  
  p3 = theCode->LastInstr();
  PatchBreaksAfter(p3);
  enclosingLoop.del_front();
  return 1;
}

virtual int RepeatLoop::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Instruction *l;
  Instruction *p1;
  Instruction *p2;
  Instruction *p3;
  int ic;
  Iteration *iter;

  iter = (Iteration *)this;
  enclosingLoop.prepend(iter);
  //get the location of the last instruction
  p1 = theCode->LastInstr();

  ic = theExpressions->emit(theCode,theContainer);
  //if there are items on the stack, pop them off
  if (ic) 
    theCode->AppendInstruction(BC_POP,0,none);  

  //emit the initial expression
  ic = untilExpression->emit(theCode,theContainer);

  p2 = theCode->AppendInstruction(BC_GOTOIFNOTNIL,0,none);  
  p2->SetPatchAfter(p1);
  p3 = theCode->AppendInstruction(BC_PUSHIMMEDIATE,VAL_NIL,unique);  
  PatchBreaksAfter(p3);
  enclosingLoop.del_front();
  return 1;
};

virtual int BreakExpr::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  Iteration *encl;
  Instruction *p1;

  encl = enclosingLoop.front();
  if (theExpression != NULL)
    theExpression->emit(theCode,theContainer);
  p1 = theCode->AppendInstruction(BC_GOTO,0,none);  
  encl->addBreak(p1);
  return 0;
}

virtual int OnExceptionClause::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in
  theObjectFile->outFile << "OnExceptionClause";
  theSymbol->emit(theCode,theContainer);
  theExpression->emit(theCode,theContainer);
  return 0;
}

virtual int TryExpr::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO this needs to be filled in 
  theObjectFile->outFile << "TryExpr";
  theExpressions->emit(theCode,theContainer);
  
  ExcClauseList tempList = theClauses;
  OnExceptionClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}

virtual int LocalDecl::emit(Codeblock *theCode,NO_Frame *theContainer)
{

  InitClauseList tempList = theInitClauses;
  InitClause *tempItem;

  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}

virtual int ConstantInitClause::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "ConstantInitClause";
  theSymbol->emit(theCode,theContainer);
  theExpression->emit(theCode,theContainer);
  return 0;
};

virtual int ConstantDecl::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  theObjectFile->outFile << "ConstantDecl ";
  
  CIClauseList tempList = theDeclClauses;
  ConstantInitClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->emit(theCode,theContainer);
    };
  return 0;
}


virtual int ReturnExpr::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //if there is an expression, emit it
  if (theExpression != NULL)
    theExpression->emit(theCode,theContainer);
  //return 
  theCode->AppendInstruction(BC_RETURN,0,none);
  return 1;  
}


virtual int NString::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  NO_String *theString;
  int litNum;

  theString = new NO_String(value);
  NO_VpumPtr *tempPtr;
  //push literal reference
  tempPtr = theString->CreateVPointer();
  litNum = theContainer->AddLiteral(tempPtr);
  theCode->AppendInstruction(BC_PUSHLITERAL,litNum,simple);
  return 1;
}

virtual int NInteger::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //TODO if the integer is big, we should make a literal, and push its value
  theCode->AppendInstruction(BC_PUSHIMMEDIATE,value,simple);
  return 1;
}

virtual int NReal::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  NO_String *theString;
  int litNum;

  //TODO need to translate real number here
  theString = new NO_String("abc");
  NO_VpumPtr *tempPtr;

  tempPtr = theString->CreateVPointer();
  litNum = theContainer->AddLiteral(tempPtr);
  theCode->AppendInstruction(BC_PUSHLITERAL,litNum,simple);
  return 1;
}

virtual int NCharacter::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  int t = value;
  //TODO convert t into a unicode character
  t = (t << 4) + 6;
  theCode->AppendInstruction(BC_PUSHIMMEDIATE,t,unique);
  return 1;
};

virtual int NTrue::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  // push the true value on the stack
  theCode->AppendInstruction(BC_PUSHIMMEDIATE,0x1A,unique);
  return 1;
};

virtual int NNil::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  // push the nil value on the stack
  theCode->AppendInstruction(BC_PUSHIMMEDIATE,0x2,unique);
  return 1;
}


virtual int MPointer::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  //push the magic value
  theCode->AppendInstruction(BC_PUSHIMMEDIATE,value,magic);
  return 1;
}



