// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#include <String.h>

#include "evalue.h"

#include <iostream.h>

void EValue::operator = (const EValue r)
{ theType = r.theType;
  switch (theType)
    {
    case EVString:
      strValue = r.strValue;
      break;
    case EVInt:
      intValue=r.intValue;
      break;
    case EVChar:
      charValue = r.charValue;
      break;
    case EVReal:
      realValue = r.realValue;
      break;
    case EVUlong:
      ulValue = r.ulValue;
    case EVNone:
      break;
    }
}

EValue::dump()
{ 
  switch (theType)
    {
    case EVString:
      cerr << strValue;
      break;
    case EVInt:
      cerr << intValue;
      break;
    case EVChar:
      cerr << charValue;
      break;
    case EVReal:
      cerr << realValue;
      break;
    case EVUlong:
      cerr << "@" << ulValue;
      break;
    case EVNone:
      break;
    }
}

EValue EValue::operator+(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVString:
	  if (r.theType == EVString)
	    retval = EValue(strValue+r.strValue);
	  break;
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(intValue+r.intValue);
	  if (r.theType == EVReal)
	    retval = EValue(intValue+r.realValue);
	  break;
	case EVReal:
	  if (r.theType == EVReal)
	    retval = EValue(realValue+r.realValue);
	  if (r.theType == EVInt)
	    retval = EValue(realValue+r.intValue);
	  break;
	}
      return retval;
    }


  EValue EValue::operator-(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(intValue-r.intValue);
	  if (r.theType == EVReal)
	    retval = EValue(intValue-r.realValue);
	  break;
	case EVReal:
	  if (r.theType == EVReal)
	    retval = EValue(realValue-r.realValue);
	  if (r.theType == EVInt)
	    retval = EValue(realValue-r.intValue);
	  break;
	}
      return retval;
    }

  EValue EValue::operator*(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(intValue*r.intValue);
	  if (r.theType == EVReal)
	    retval = EValue(intValue*r.realValue);
	  break;
	case EVReal:
	  if (r.theType == EVReal)
	    retval = EValue(realValue*r.realValue);
	  if (r.theType == EVInt)
	    retval = EValue(realValue*r.intValue);
	  break;
	}
      return retval;
    }

  EValue EValue::operator/(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(((double)intValue)/((double)r.intValue));
	  if (r.theType == EVReal)
	    retval = EValue(((double)intValue)/((double)r.realValue));
	  break;
	case EVReal:
	  if (r.theType == EVReal)
	    retval = EValue(realValue/r.realValue);
	  if (r.theType == EVInt)
	    retval = EValue(realValue/r.intValue);
	  break;
	}
      return retval;
    }

  EValue EValue::div(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(intValue/r.intValue);
	  if (r.theType == EVReal)
	    retval = EValue(intValue/(int)r.realValue);
	  break;
	case EVReal:
	  if (r.theType == EVReal)
	    retval = EValue((int)(realValue/r.realValue));
	  if (r.theType == EVInt)
	    retval = EValue((int)(realValue/r.intValue));
	  break;
	}
      return retval;
    }


  EValue EValue::mod(const EValue r) const
    {
      EValue retval;

      switch (theType)
	{
	case EVInt:
	  if (r.theType == EVInt)
	    retval = EValue(intValue%r.intValue);
	  break;
	}
      return retval;
    }


  EValue EValue::lshft(const EValue r) const
    {
      EValue retval;

      if ((theType == EVInt) && (r.theType == EVInt))
	retval = EValue(intValue << r.intValue);
      return retval;
    }

  EValue EValue::rshft(const EValue r) const
    {
      EValue retval;

      if ((theType == EVInt) && (r.theType == EVInt))
	retval = EValue(intValue >> r.intValue);
      return retval;
    }

  EValue EValue::band(const EValue r) const
    {
      EValue retval;

      if ((theType == EVInt) && (r.theType == EVInt))
	retval = EValue(intValue & r.intValue);
      return retval;
    }

  EValue EValue::bor(const EValue r) const
    {
      EValue retval;

      if ((theType == EVInt) && (r.theType == EVInt))
	retval = EValue(intValue | r.intValue);
      return retval;
    }

  EValue EValue::uminus() const
    {
      EValue retval;

      if (theType == EVInt)
	{
	  retval = EValue( -intValue);
	}
      else
	{
	  if (theType == EVReal)
	    retval = EValue( -realValue);
	}
      
      return retval;
    }

  EValue EValue::not() const
    {
      EValue retval;

      if (theType == EVInt)
	{
	  retval = EValue( ~intValue);
	}
      return retval;
    }

