%{
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



 *
 */
#include <assert.h>
#include "prod.h" 
#include <stdlib.h>
#include <string.h>
#include "symtab.h"
#include <iostream.h>
/*
   #undefine token(x) (int) "x"
*/

static int yyinput();
char linebuf[1000];
#include "y.tab.h"

int yylex( void );

int s_lookup(int yylex);

extern Symtab theSymbolTable;
extern Symtab *currentSymtab;
extern int lineno;

#define NUMELEMENTS(v) (sizeof (v) / sizeof (v[0]))
static int screen();

#define token(xx) xx
#define is(xx)  return xx 

void add_nls(char *pt);


%}


HEX_DIGIT  [0-9a-fA-F]
DIGIT      [0-9]
ALPHA      [a-zA-Z]
NON_ESC_CHAR [\040-\133\135-\177]
CHARACTER  \$({NON_ESC_CHAR}|\\(\\|n|t|({HEX_DIGIT}{HEX_DIGIT}))|(u{HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT}))
real      {DIGIT}+\.{DIGIT}*((e|E)-?{DIGIT}+)?
integer   (({DIGIT}+)|(0x{HEX_DIGIT}+))

SYMBOL_CHAR [\040-\133\135-\173\175-\177]
symbol ({ALPHA}|_)({ALPHA}|{DIGIT}|_)*|(\|({SYMBOL_CHAR}|(\\(\||\\)))*\|)
TRUNCATED_ESCAPE \\u({HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT})*
ESCAPE_SEQUENCE (\\(\"|\\|n|t))|(\\u({HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT}{HEX_DIGIT})*\\u) 
STRING_CHARACTER [\011\040\041\043-\133\135-\177]
CHARACTER_SEQUENCE   ({STRING_CHARACTER}|{ESCAPE_SEQUENCE})*{TRUNCATED_ESCAPE}?
string    \"{CHARACTER_SEQUENCE}\"

  
white_space  [ \t]
blank        [ \t]
other      .
all_white    [ \t\n]
nl           \n

sem_else ";"{all_white}*"else" 
sem_end  ";"{all_white}*"end"
sem_until ";"{all_white}*"until"
sem_onexcept ";"{all_white}*"onexception"

sem_comma ";"{all_white}*"," 
comma_brack ","{all_white}*"]"
comma_brace ","{all_white}*"}"

cmt1_start "/*"
cmt2_start "//"

semi_cmt1_start ";"{all_white}*{cmt1_start}
semi_cmt2_start ";"{all_white}*{cmt2_start}

%%

{cmt1_start}  {int c1=0,c2=yyinput();
       for(;;) {
         if(c2 == EOF)
           break;
         if(c2 == '\n')
           ++lineno;
         if(c1 == '*' && c2 == '/')
           break;
         c1 = c2;
          c2 = yyinput();
         }
       if (c2 == EOF)
          unput(EOF);
       else
          unput(' ');
      }

{cmt2_start}  {int c2=yyinput();
       for(;;) {
         if(c2 == EOF)
           break;
         if(c2 == '\n')
          {
           ++lineno;
           break;
          }
         c2 = yyinput();
         }
       if (c2 == EOF)
          unput(EOF);
       else
          unput(' ');
      }


{semi_cmt1_start}  {int c1=0,c2=yyinput();
       for(;;) {
         if(c2 == EOF)
           break;
         if(c2 == '\n')
           ++lineno;
         if(c1 == '*' && c2 == '/')
           break;
         c1 = c2;
          c2 = yyinput();
         }
       if (c2 == EOF)
          unput(EOF);
       else
          unput(';');
      }

{semi_cmt2_start}  {int c2=yyinput();
       for(;;) {
         if(c2 == EOF)
           break;
         if(c2 == '\n')
	 {
           ++lineno;
           break;
	   }
         c2 = yyinput();
         }
       if (c2 == EOF)
          unput(EOF);
       else
          unput(';');
      }



@{DIGIT}+	{ s_lookup(token(Mpointer));return token(Mpointer);}

"&&"  is(AA);
"<<"  is(LS);
">>"  is(RS);
"<>"  is(NE);
"<="  is(LE);
">="  is(GE);
":="  is(ASNMT);
":?"  is(CQ);
"-"   is(MINUS);

{sem_else} {add_nls(yytext);is(SEMI_ELSE);}
{sem_end} {add_nls(yytext);is(SEMI_END);}
{sem_until} {add_nls(yytext);is(SEMI_UNTIL);}
{sem_onexcept} {add_nls(yytext);is(SEMI_ONEXCEPTION);}
{comma_brack} {add_nls(yytext);is(COMMA_BRACK);}
{comma_brace} {add_nls(yytext);is(COMMA_BRACE);}
{sem_comma} {add_nls(yytext);is(SEM_COMMA);}
"and"  is(AND_TOK);
"begin"  is(BEGIN_TOK);
"break"  is(BREAK);
"by"  is(BY);
"call"  is(CALL);
"constant"  is(CONSTANT);
"collect"  is(COLLECT);
"div"  is(DIV);
"do"  is(DO);
"deeply"  is(DEEPLY);
"else"  is(ELSE);
"end"  is(END);
"exists"  is(EXISTS);
"for"  is(FOR);
"foreach"  is(FOREACH);
"func"  is(FUNC);
"global"  is(GLOBAL);
"if"  is(IF);
"in"  is(IN);
"inherited"  is(INHERITED);
"local"  is(LOCAL);
"loop"  is(LOOP);
"mod"  is(MOD);
"MOD"  is(MOD);
"nil"  {NNil *newNil = new NNil; yylval.theNil = newNil;is(NIL);}
"NIL"  {NNil *newNil = new NNil; yylval.theNil = newNil;is(NIL);}
"not"  is(NOT);
"NOT"  is(NOT);
"or"  is(OR);
"OR"  is(OR);
"onexception"  is(ONEXCEPTION);
"repeat"  is(REPEAT);
"return"  is(RETURN);
"self"  is(SELF);
"then"  is(THEN);
"to"  is(TO);
"true"   {NTrue *newTrue = new NTrue; yylval.theTrue = newTrue;is(TRUE);}
"try"  is(TRY);
"until"  is(UNTIL);
"while"  is(WHILE_TOK);
"with"  is(WITH);



{symbol}	{   String ss(yytext);	yylval.theSymbol = new Symbol(ss);is(Symbol_Tok);}

{string}	{s_lookup(token(String_Tok));is(String_Tok);}

{real}		{s_lookup(token(Real));is(Real);}



{integer}	{s_lookup(token(Integer));is(Integer);}

{CHARACTER}	{s_lookup(token(Character));is(Character);}

{white_space}+            ;
{nl}                 {++lineno;}
{other}                    is(yytext[0]);
%%


/*
 *
 */


int char_value(char *tt)
{
  int retval;
  switch (tt[1])
    {
    case '\\':
      switch (tt[2])
	{
	case '\\':
          retval = '\\';
          break;
        case 'n':
	  retval = 12; //TODO is this the right val for newline
          break;
        case 't':
	  retval = 7; //TODO is this the right  val for tab
          break;
	default://hex digits
	  unsigned long int ul;
	  ul = strtoul(&tt[2],NULL,16);
	  retval = (int)ul;
	  break;
        }
      break;

    case 'u':
      unsigned long int ul;
      ul = strtoul(&tt[2],NULL,16);
      retval = (int)ul;
      break;
    default:
      retval = tt[1];
      break;
    }
  return retval;
}

int s_lookup(int yylex)
{
  extern char *yytext;
  String ss(yytext);	
  switch (yylex)
    {
    case String_Tok:  //TODO this needs to decode escape-sequences
      char *ts = strdup(yytext);
      ts[strlen(ts)-1] = 0;
      yylval.theString = new NString(strdup(&ts[1]));
      free(ts);
      break;
    case Symbol_Tok:
      yylval.theSymbol = new Symbol(ss);
      break;
    case Integer:
      unsigned long ul;
      ul = strtoul(yytext,NULL,0);
      yylval.theInteger = new NInteger((int)ul);
      break;
    case Real:
      double dd;
      dd = strtod(yytext,NULL);
      yylval.theReal = new NReal(dd);
      break;
    case Character:
      yylval.theCharacter = new NCharacter(char_value(yytext));
      break;
    case Mpointer:
      unsigned long ulm;
      ulm = strtoul(&yytext[1],NULL,0);
      yylval.theMPointer = new MPointer(ulm);
    }
}


yymark()
  {
  }


int screen()
{
  extern char *yytext;
  int c;
  int elements;

  s_lookup(token(Symbol_Tok));
  is(Symbol_Tok);
}

void add_nls(char *pt)
{
  int i,ll;
  ll = strlen(pt);
  for (i=0;i<ll;++i)
       {
       if (pt[i] == '\n')
         ++lineno;
       }
}
