// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#include <stdio.h>
#include <assert.h>
#include <fstream.h>
#include <stdlib.h>
#include "objects.h"
#include <GetOpt.h>
#include "newtobjs.h"


#define DEFAULT_OUTFILE "newtonsc.out"
int error(char *,char *);
extern FILE *yyin,*yyout;
Input *globalInput;
extern int yyparse();
Codeblock globalCode;
NO_Frame *globalFrame;

int tree_debug = 0;
int symtab_debug = 0;
int object_debug = 0;
int canrun = 0;

int yyerror(char *s)
  {
    error(s,NULL);
  }

Symtab globalSymbolTable;
char *sourcefile;
int lineno = 1;
ObjectFile *theObjectFile;


init()
{
  theObjectFile->Open();
}

finish()
{
  allObjects.setDebugMode(object_debug);
  allObjects.dump();
  if (symtab_debug)
    globalSymbolTable.dump("Global");
  theObjectFile->Close();
}

usage(char *argv[])
{
  fprintf (stderr,
	   "usage: %s [-d(ly) -o<outfile> <input file>]\n", argv[0]);
}


main(int argc,char *argv[])
{
  char *p; 
  char *infile;
  char *outfile;
  char *progname;
  
  extern int yydebug;
  extern int yy_flex_debug;
  
  GetOpt getopt(argc,argv,"d:o:i:");
  int option_char;

  yydebug = 0;
  yy_flex_debug = 0;
  tree_debug = 0;
  symtab_debug = 0;
  object_debug = 0;
  
  // Invokes member function `int operator ()(void);'
  while ((option_char = getopt ()) != EOF)
    switch (option_char) 
      {
      case 'd':
	switch (getopt.optarg[0])
	  {
	  case 'y':
	    yydebug = 1;
	    break;
	  case 'l':
	    yy_flex_debug = 1;
	    break;
	  case 't':
	    tree_debug = 1;
	    break;
	  case 's':
	    symtab_debug = 1;
	    break;
	  case 'o':
	    object_debug = 1;
	    break;
	  }
	break;
      case 'o':
	outfile = strdup(getopt.optarg);
	break;
      case 'i':
	canrun = 1;
	infile = getopt.optarg;
	break;
      case '?': 
	usage(argv);
	exit(0);
	break;
      }
  if (!canrun)
    {
      usage(argv);

      exit(0);
    }
  assert(sizeof(int) >= sizeof(char *));
  progname = argv[0];
//  infile = getopt.optarg;
  sourcefile = infile;
  

  yyin = fopen(infile,"r");
  if (yyin == NULL)
    {
      fprintf(stderr,"%s: cannot open %s\n",progname,infile);
      exit(1);
    }

  if (!outfile)
    {
      outfile = DEFAULT_OUTFILE;
    }
  theObjectFile = new ObjectFile(outfile);

  cout << "+++++  init" << endl;
  init();
  cout << "+++++  yyparse" << endl;
  yyparse();
  cout << "+++++  output" << endl;
  if (tree_debug)
    globalInput->output(0);
  cout << "+++++  walk" << endl;  
  globalInput->walk(&globalSymbolTable);
  cout << "+++++  emit" << endl;
  globalFrame = new NO_Frame;
  globalInput->emit(&globalCode,globalFrame);
  cout << "+++++  finish" << endl;
  finish();
}


warning(char *s,char *t)
{
  fprintf(stderr,"%s:%d: warning:%s",sourcefile,lineno,s);
  if (t)
    fprintf(stderr," %s",t);
  fprintf(stderr,"\n");
}

error(char *s,char *t)
{
  fprintf(stderr,"%s:%d: %s",sourcefile,lineno,s);
  if (t)
    fprintf(stderr," %s",t);
  fprintf(stderr,"\n");
  exit(1);
}






