#ifndef INCL_NEWTOBJS_H
#define INCL_NEWTOBJS_H
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/

#include <DLList.h>
#include <iostream.h>
#include <String.h>
class NO_Object;

typedef DLList<NO_Object *> NO_ObjectList;

class AllObjects
{
  NO_ObjectList theObjectList;
  int object_debug;
 public:
  AddObject(NO_Object *newobj)
    {
      theObjectList.append(newobj);
    }
  dump();
  setDebugMode(int dmode) {object_debug = dmode;}
};

extern AllObjects allObjects;

class NO_VpumPtr;

class NO_Object
{
  unsigned long offset;
 public:
  NO_Object()
    {
      allObjects.AddObject(this);
    }
  virtual int dump(int tl) = 0;
  void outtab(int c);
  virtual unsigned long getHash() = 0;

  NO_VpumPtr *CreateVPointer();

  virtual String GetString()
    {
      String t;
      return t;
    }
};
 
enum NO_Type {NOT_None};



class NO_Data : public NO_Object
{
  String theString;
  int isCode;
 public:
  NO_Data(String nv):theString(nv),isCode(0){}
  NO_Data(char *sp)
    {
      theString = String(sp);
    }
  virtual int dump(int tl);
  int SetCode() {isCode = 1;}
  virtual unsigned long getHash()
    {
      return 0;
    }

};

class NO_String : public NO_Object
{
  String theString;
 public:
  NO_String(String nv):theString(nv){}
  NO_String(char *sp)
    {
      theString = String(sp);
    }
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "String:" << theString ;
    }
  virtual unsigned long getHash()
    {
      return 0;
    }
};

class NO_Symbol : public NO_Object
{
  unsigned long symbolHashValue;
  String symbolName;

 public:
  virtual unsigned long getHash()
    {
      return symbolHashValue;
    }

  ComputeHash()
    {
      int temph = 0;
      String ts = symbolName;

      ts.upcase();
      for (int i=0;i<ts.length();++i)
	{
	  temph += (int)ts[i];
	}
      temph = temph * 0x9e3779b9;
      symbolHashValue = temph;
    }
  
  NO_Symbol(String &nv)
    {
      symbolName = nv;
      ComputeHash();
    }  
  NO_Symbol(char *sp)
    {
      symbolName = String(sp);
      ComputeHash();
    }  
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "Symbol:" << symbolName << ":";
      cout.form("0x%lx",symbolHashValue);
    }
  virtual String GetString()
    {
      return symbolName;
    }
};




class NO_Vpum : public NO_Object 
{
 public:
  virtual int dump()
    {
      };
  virtual unsigned long getHash() = 0;
};

class NO_VpumInt : public NO_Vpum
{
  int theValue;
 public:
  NO_VpumInt(int nv):theValue(nv){}
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "VpumInt:" << theValue;
    }
  virtual unsigned long getHash()
    {
      return 0;
    }
};

class NO_VpumPtr : public NO_Vpum
{
  unsigned long theOffset;
  NO_Object *theValue;
 public:
  NO_VpumPtr(NO_Object *nob):theValue(nob){}
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "VpumPtr: " << theOffset;
      theValue->dump(tl+4);
    }
  String GetString()
    {
      return theValue->GetString();
    }
  virtual unsigned long getHash()
    {
      return theValue->getHash();
    }
};

class NO_VpumUnusual : public NO_Vpum
{
  int theValue;
 public:
  NO_VpumUnusual(int nv):theValue(nv){}
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "VpumUnusual:" << theValue;
    }
  virtual unsigned long getHash()
    {
      return 0;
    }

};

class NO_VpumMagic : public NO_Vpum
{
  unsigned long theValue;
 public:
  NO_VpumMagic(unsigned long ul):theValue(ul){}
  virtual int dump(int tl)
    {
      outtab(tl);
      cout << "VpumMagic:" << theValue;
    }
  virtual unsigned long getHash()
    {
      return 0;
    }

};



class NO_Array : public NO_Object
{
  NO_Type theType;
  NO_ObjectList theElements;  
 public:
  NO_Array(NO_Type nt) {theType=nt;}

  int AddElement(NO_Object *ob) 
    {
      theElements.append(ob);
      return theElements.length();
    }
  virtual int dump(int tl);
  dumpElement(int eno,int tl);
  unsigned long getElementHash(int eno);
  virtual unsigned long getHash()
    {
      return 0;
    }
};



class NO_Frame : public NO_Object
{
  NO_ObjectList theSlots;
  NO_Array *frameMap;
  int isFunctionFrame;
  NO_Array *litArray;
 public:
  NO_Frame(){isFunctionFrame = 0;frameMap = new NO_Array(NOT_None);}
  AddSlot(NO_Symbol *s,NO_Object *ob);
  SetFunctionFrame(NO_Array *theLits) {isFunctionFrame = 1;litArray = theLits;}
  int dump(int tl);
  int AddLiteral(NO_Object *nob)
    {
      if (isFunctionFrame)
	return litArray->AddElement(nob);
      else
	return -1;
    }
  virtual unsigned long getHash()
    {
      return 0;
    }
};



#endif
