%{
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/


#include "objectfi.h"

#include "prod.h"
#include "symtab.h"
#include "newtobjs.h"
#include "operator.h"
#include <assert.h>
int yyerror(char *s);
int yylex( void );

extern warning(char *s,char *t);
extern error(char *s,char *t);
extern Input *globalInput;
extern Symtab globalSymbolTable;
%}




%union {
  Constituent *theConstituent;
  Input *theInput;
  Expression *theExpression;
  ExpressionSequence *theExpressionSequence;
  Lvalue *theLvalue;
  Symbol *theSymbol;
  FormalArgumentList *theFormalArgumentList;
  InitClause *theInitClause;
  GlobalDeclaration *theGlobalDeclaration;
  GlobalFunctionDecl *theGlobalFunctionDecl;
  GlobalWithInitialization *theGlobalWithInitialization;
  SimpleExpression *theSimpleExpression;
  CompoundExpression *theCompoundExpression;
  Literal *theLiteral;
  SimpleLiteral *theSimpleLiteral;
  Object *theObject;
  SimpleObject *theSimpleObject;
  PathExpression *thePathExpression;
  PathObject *thePathObject;
  Array *theArray;
  ArrayObject *theArrayObject;
  FrameSlot *theFrameSlot;
  Frame *theFrame;
  FrameObject *theFrameObject;
  Constructor *theConstructor;
  ArrayConstructor *theArrayConstructor;
  FrameConstructorSlot *theFrameConstructorSlot;
  FrameConstructor *theFrameConstructor;
  FunctionConstructor *theFunctionConstructor;
  FrameAccessor *theFrameAccessor;
  ArrayAccessor *theArrayAccessor;
  Assignment *theAssignment;
  ExistsExpr *theExistsExpr;
  SymbolExists *theSymbolExists;
  FrameAccExists *theFrameAccExists;
  ExprExists *theExprExists;
  ActualArgumentList *theActualArgumentList;
  FunctionCall *theFunctionCall;
  MessageSend *theMessageSend;
  IfExpression *theIfExpression;
  Iteration *theIteration;
  InfiniteLoop *theInfiniteLoop;
  ForLoop *theForLoop;
  ForEachLoop *theForEachLoop;
  WhileLoop *theWhileLoop;
  RepeatLoop *theRepeatLoop;
  BreakExpr *theBreakExpr;
  OnExceptionClause *theOnExceptionClause;
  TryExpr *theTryExpr;
  LocalDecl *theLocalDecl;
  ConstantInitClause *theConstantInitClause;
  ConstantDecl *theConstantDecl;
  ReturnExpr *theReturnExpr;
  NString *theString;
  NInteger *theInteger;
  NReal *theReal;
  MPointer *theMPointer;
  NCharacter *theCharacter;
  NTrue *theTrue;
  NNil *theNil;
  int int_flag;
}

%token <theString> String_Tok
%token <theSymbol> Symbol_Tok
%token <theInteger> Integer
%token <theReal> Real
%token <theCharacter> Character
%token <theMPointer> Mpointer
%token AND_TOK
%token BEGIN_TOK
%token BREAK
%token BY
%token CALL
%token COLLECT
%token CONSTANT
%token DEEPLY
%token DIV
%token DO
%token ELSE
%token SEMI_ELSE
%token END
%token SEMI_END
%token EXISTS
%token FOR
%token FOREACH
%token FUNC
%token GLOBAL
%token IF
%token IN
%token INHERITED
%token LOCAL
%token LOOP
%token MOD
%token MINUS
%token <theNil> NIL
%token NOT
%token ONEXCEPTION
%token SEMI_ONEXCEPTION
%token OR
%token REPEAT
%token RETURN
%token SELF
%token THEN
%token TO
%token <theTrue> TRUE
%token TRY
%token UNTIL
%token SEMI_UNTIL
%token WHILE_TOK
%token WITH
%token COMMA_BRACK
%token COMMA_BRACE
%token SEM_COMMA

%token ';'
%token ')'
%token '('

%token AA /* && */
%token LS /* << */
%token RS /* >> */
%token NE /* <> */
%token LE /* <= */
%token GE /* >= */
%token ASNMT /* := */
%token CQ /* :? */


%right ASNMT
%left AND_TOK OR
%left NOT
%left '<' LE '>' GE '=' NE
%nonassoc EXISTS
%left '&' AA
%left '+' MINUS
%left '*' '/' DIV MOD
%left RS LS
%left '[' ']'
%left ':' CQ
%left '.'
%nonassoc ','

%type <theInput>   input
%type <theConstituent>   constituent
%type <theExpression>   expression
%type <theExpressionSequence>   expression_sequence
%type <theLvalue>   lvalue
%type <theFormalArgumentList>   formal_argument_list
%type <theFormalArgumentList>   optional_formal_argument_list
%type <theInitClause>   initialization_clause
%type <theGlobalDeclaration>   global_declaration
%type <theSimpleExpression>   simple_expression
%type <theCompoundExpression>   compound_expression
%type <theLiteral>   literal
%type <theSimpleLiteral>   simple_literal
%type <theObject>   object
%type <thePathExpression>   path_expression
%type <theArray>   array
%type <theFrameSlot>   frame_slot
%type <theFrame>   frame
%type <theConstructor>   constructor
%type <theArrayConstructor>   array_constructor
%type <theFrameConstructorSlot>   frame_constructor_slot
%type <theFrameConstructor>   frame_constructor
%type <theFunctionConstructor>   function_constructor
%type <theFrameAccessor>   frame_accessor
%type <theArrayAccessor>   array_accessor
%type <theAssignment>   assignment

%type <theActualArgumentList>   actual_argument_list
%type <theFunctionCall>   function_call


%type <theIfExpression>   if_expression
%type <theIteration>   iteration
%type <theInfiniteLoop>   infinite_loop
%type <theForLoop>   for_loop
%type <theForEachLoop>   foreach_loop
%type <theWhileLoop>   while_loop
%type <theRepeatLoop>   repeat_loop
%type <theBreakExpr>   break_expression
%type <theOnExceptionClause>   onexception_clause
%type <theTryExpr>   try_expression
%type <theLocalDecl>   local_declaration
%type <theConstantInitClause>   constant_init_clause
%type <theConstantDecl>   constant_declaration
%type <theReturnExpr>   return_expression
%type <int_flag> foreach_do_or_collect
%type <int_flag> optional_deeply
%type <theInput> constituent_list
%type <theSymbol> optional_array_Symbol_Tok
%type <theArray> array_expression_list
%type <theArray> optional_array_expression_list

%type <theFrame>   optional_frame_slot_list
%type <theFrame>   frame_slot_list
%type <theConstantDecl> constant_init_clause_list
%type <theLocalDecl> initialization_clause_list
%type <theTryExpr> onexception_list

%type <theActualArgumentList>   optional_actual_argument_list

%type <theFrameConstructor>   frame_constructor_slot_list
%type <theFrameConstructor>   optional_frame_constructor_slot_list
%type <theSymbol> foreach_optional_symbol

%type <theArrayConstructor> array_constructor_exp_list
%type <theArrayConstructor> optional_array_constructor_exp_list



%%

input
	: constituent_list 
            {
	      globalInput = $1;
	    }
	| constituent_list ';'
            {
	      globalInput = $1;
	    }
	;

constituent_list
	: constituent_list ';' constituent { $$ = $1->AddConstituent($3)}
	| constituent  { $$ = new Input($1)}
	;

constituent
	: expression { $$ = $1;} 
	| global_declaration { $$ = $1;}
	;

expression
	: simple_expression { $$ = $1 }
	| compound_expression  { $$ = $1 }
	| literal  { $$ = $1 }
	| constructor  { $$ = $1 }
	| lvalue    { $$ = $1 }
	| assignment  { $$ = $1 }
	| msgsend_or_exists  { }
	| function_call  { $$ = $1 }
	| if_expression  { $$ = $1 }
	| iteration { $$ = $1 }
	| break_expression  { $$ = $1 }
	| try_expression  { $$ = $1 }
	| local_declaration  { $$ = $1 }
 	| constant_declaration  { $$ = $1 }
	| return_expression  { $$ = $1 }
	;
 
simple_expression
	: expression '+' expression     {$$= new SimpleExpression($1,  $3,   OPER_PLUS  );}
	| expression MINUS expression     {$$= new SimpleExpression($1,  $3,   OPER_MINUS );}
	| expression '*' expression     {$$= new SimpleExpression($1,  $3,   OPER_MUL   );}
	| expression '/' expression     {$$= new SimpleExpression($1,  $3,   OPER_FDIV  );}
	| expression DIV expression     {$$= new SimpleExpression($1,  $3,   OPER_IDIV  );}
	| expression MOD expression     {$$= new SimpleExpression($1,  $3,   OPER_MOD   );}
	| expression LS  expression     {$$= new SimpleExpression($1,  $3,   OPER_LS    );}
	| expression RS  expression     {$$= new SimpleExpression($1,  $3,   OPER_RS    );}
	| expression '=' expression     {$$= new SimpleExpression($1,  $3,   OPER_EQ    );}
	| expression NE expression      {$$= new SimpleExpression($1,  $3,   OPER_NE    );}
	| expression '<' expression     {$$= new SimpleExpression($1,  $3,   OPER_LT    );}
	| expression '>' expression     {$$= new SimpleExpression($1,  $3,   OPER_GT    );}
	| expression LE expression      {$$= new SimpleExpression($1,  $3,   OPER_LE    );}
	| expression GE expression      {$$= new SimpleExpression($1,  $3,   OPER_GE    );}
	| expression AND_TOK expression {$$= new SimpleExpression($1,  $3,   OPER_AND   );}
	| expression OR expression      {$$= new SimpleExpression($1,  $3,   OPER_OR    );}
   	| expression '&' expression     {$$= new SimpleExpression($1,  $3,   OPER_BAND  );}
	| expression AA expression      {$$= new SimpleExpression($1,  $3,   OPER_AA    );}
	| MINUS expression                {$$= new SimpleExpression($2,  NULL, OPER_UMINUS);}
	| NOT expression                {$$= new SimpleExpression($2,  NULL, OPER_UNOT  );}
	| '(' expression ')'            {$$= new SimpleExpression($2,  NULL, 0          );}
	| SELF                          {$$= new SimpleExpression(NULL,NULL, OPER_SELF  );}
	;



compound_expression
	: BEGIN_TOK expression_sequence SEMI_END  { $$ = new CompoundExpression($2);}
	| BEGIN_TOK expression_sequence END  { $$ = new CompoundExpression($2);}
	| BEGIN_TOK END  { $$ = new CompoundExpression(NULL);}
	;

expression_sequence
	: expression_sequence ';' expression
	      {$$ = $1->AddExpression($3); }
	| expression 
	    {$$ = new ExpressionSequence($1);}
        ;

literal
	:  simple_literal {$$ = $1}
	| '\'' object {$$ = $2}
	;

simple_literal
	: String_Tok { $$ = (SimpleLiteral *)$1}
	| Integer  { $$ = (SimpleLiteral *)$1}
	| Real  { $$ = (SimpleLiteral *)$1}
	| Character { $$ = (SimpleLiteral *)$1}
	| Mpointer { $$ = (SimpleLiteral *)$1}
	| TRUE { $$ = (SimpleLiteral *)$1}
	| NIL { $$ = (SimpleLiteral *)$1}
	;

object
	: simple_literal { $$ = new SimpleObject($1)}
	| path_expression {$$ = new PathObject($1)}
	| array {$$ = new ArrayObject($1)}
	| frame {$$ = new FrameObject($1)}
	;

path_expression
	: Symbol_Tok {$$ = new PathExpression($1); }
	| path_expression '.' Symbol_Tok { $$ = $1->AddSymbol($3);}
	;

/* NOTE: Each dot in ' Symbol_Tok . Symbol_Tok ... is ambiguous: it could be a continuation of the path expression, or a slot accessor.  NewtonScript uses the first interpretation: 'x.y.z is one long path expression, not ('x).y.z.
*/

array
	: '[' optional_array_Symbol_Tok optional_array_expression_list COMMA_BRACK
             { if ($3 == NULL) 
                 { $$ = new Array($2);}
               else
                 {$$ = $3->SetSymbol($2);}
             }
	| '[' optional_array_Symbol_Tok optional_array_expression_list ']'
             { if ($3 == NULL) 
                 { $$ = new Array($2);}
               else
                 {$$ = $3->SetSymbol($2);}
             }
	;

optional_array_expression_list
	: {$$ = NULL;}
	| array_expression_list { $$ = $1}
	;

array_expression_list
	: object {$$ = new Array($1)}
	| array_expression_list ',' object {$$ = $1->AddObject($3);}  
	;

optional_array_Symbol_Tok
	: {$$ = NULL;}
	| Symbol_Tok ':' {$$ = $1}
	;


frame
	: '{' optional_frame_slot_list COMMA_BRACE
               { if ($2 == NULL)
                  { $$ = new Frame();}
                 else
                  { $$ = $2;}
               }
	| '{' optional_frame_slot_list '}' 
               { if ($2 == NULL)
                  { $$ = new Frame();}
                 else
                  { $$ = $2;}
               }
	;

optional_frame_slot_list
	: {$$ = NULL}
	| frame_slot_list {$$ = $1}
	;

frame_slot_list
	: frame_slot { $$ = new Frame($1);}
	| frame_slot_list SEM_COMMA frame_slot {$$ = $1->AddSlot($3);}
	| frame_slot_list ',' frame_slot {$$ = $1->AddSlot($3);}
	;

frame_slot
	: Symbol_Tok ':' object {$$ = new FrameSlot($1,$3);}
	;

constructor
	: array_constructor { $$ = $1;}
	| frame_constructor { $$ = $1;}
	| function_constructor { $$ = $1;}
	;

array_constructor
	: '[' Symbol_Tok ':' optional_array_constructor_exp_list COMMA_BRACK
{
  if ($4 == NULL)
    { $$ = new ArrayConstructor($2);}
  else
    { $$ = $4->SetSymbol($2);}
}
	| '[' optional_array_constructor_exp_list COMMA_BRACK
{
  if ($2 == NULL)
    { $$ = new ArrayConstructor;}
  else
    { $$ = $2;}
}
	| '[' Symbol_Tok ':' optional_array_constructor_exp_list ']' 
{
  if ($4 == NULL)
    { $$ = new ArrayConstructor($2);}
  else
    { $$ = $4->SetSymbol($2);}
}
	| '[' optional_array_constructor_exp_list ']' 
{
  if ($2 == NULL)
    { $$ = new ArrayConstructor;}
  else
    { $$ = $2;}
}
	;

optional_array_constructor_exp_list
	: {$$ = NULL;}
	| array_constructor_exp_list { $$ = $1;}
	;

array_constructor_exp_list
	: expression { $$ = new ArrayConstructor($1);}
	| array_constructor_exp_list ',' expression { $$ = $1->AddExpression($3);}
	;


/*
NOTE:	'[' Symbol_Tok : Symbol_Tok ( 
 is ambiguous: the first Symbol_Tok could be a class for the array, or a variable to be used as the receiver for a message send.  NewtonScript uses the first interpretation.
*/


frame_constructor
	: '{' optional_frame_constructor_slot_list COMMA_BRACE 
{
  if ($2 == NULL)
    { $$ = new FrameConstructor();}
  else
    { $$ = $2;}
}
	| '{' optional_frame_constructor_slot_list '}' 
{
  if ($2 == NULL)
    { $$ = new FrameConstructor();}
  else
    { $$ = $2;}
}
	;

optional_frame_constructor_slot_list
	:  {$$ = NULL;}
	| frame_constructor_slot_list { $$ = $1;}
	;

frame_constructor_slot_list
	: frame_constructor_slot { $$ = new FrameConstructor($1);}
	| frame_constructor_slot_list SEM_COMMA frame_constructor_slot { $$ = $1->AddSlot($3);}
	| frame_constructor_slot_list ',' frame_constructor_slot { $$ = $1->AddSlot($3);}
	;

frame_constructor_slot
	: Symbol_Tok ':' expression {$$ = new FrameConstructorSlot($1,$3);}
	;

function_constructor
	: FUNC '(' optional_formal_argument_list ')'  expression 
{ $$ = new FunctionConstructor($3,$5,&globalSymbolTable);}
	;

optional_formal_argument_list
	: {$$ = NULL;}
	| formal_argument_list { $$ = $1;}
	;

formal_argument_list
	: Symbol_Tok { $$ = new FormalArgumentList($1);}
	| formal_argument_list ',' Symbol_Tok { $$ = $1->AddSymbol($3);}
	;

lvalue
	: Symbol_Tok  { $$ = $1;}
	| frame_accessor { $$ = $1;}
	| array_accessor { $$ = $1;}
	;

frame_accessor
	: expression '.' Symbol_Tok { $$ = new FrameAccessor($1,$3,NULL);}
	| expression '.' '(' expression ')' { $$ = new FrameAccessor($1,NULL,$4);}
	;

array_accessor
	: expression '[' expression ']' { $$ = new ArrayAccessor($1,$3);}
	;

assignment
	: lvalue ASNMT expression { $$ = new Assignment($1,$3);}
	;

function_call
	: Symbol_Tok '(' optional_actual_argument_list ')' 
              { $$ = new FunctionCall(0,$1,NULL,$3);}
	| CALL expression WITH '(' optional_actual_argument_list ')' 
              { $$ = new FunctionCall(1,NULL,$2,$5);}
	;

optional_actual_argument_list
	: {$$ = NULL;}
	| actual_argument_list {$$ = $1}
	;

actual_argument_list
	: expression { $$ = new ActualArgumentList($1);}
	| actual_argument_list ',' expression { $$ = $1->AddArg($3);}
	;

msgsend_or_exists
	: Symbol_Tok EXISTS { $<theExistsExpr>$ = new SymbolExists($1,0);}
	| frame_accessor EXISTS { $<theExistsExpr>$ = new FrameAccExists($1);}
	| expression ':' Symbol_Tok '(' optional_actual_argument_list ')'
             { $<theMessageSend>$ = new MessageSend(0,$1,0,$3,$5);}
	| expression ':' Symbol_Tok  EXISTS { $<theExistsExpr>$ = new ExprExists($1,$3);}
	| INHERITED  ':' Symbol_Tok '(' optional_actual_argument_list ')' 
             { $<theMessageSend>$ = new MessageSend(1,NULL,0,NULL,$5);}
	| ':' Symbol_Tok '(' optional_actual_argument_list ')'
             { $<theMessageSend>$ = new MessageSend(0,NULL,0,$2,$4);}
	| ':' Symbol_Tok EXISTS {$<theExistsExpr>$ = new SymbolExists($2,0);}
	| CQ Symbol_Tok '(' optional_actual_argument_list ')' 
             { $<theMessageSend>$ = new MessageSend(0,NULL,1,$2,$4);}
	| expression CQ Symbol_Tok '(' optional_actual_argument_list ')' 
             { $<theMessageSend>$ = new MessageSend(0,$1,1,$3,$5);}
	| INHERITED CQ Symbol_Tok '(' optional_actual_argument_list ')' 
             { $<theMessageSend>$ = new MessageSend(1,NULL,1,NULL,$5);}
	;

if_expression
	: IF expression THEN expression SEMI_ELSE expression
{ $$ = new IfExpression($2,$4,$6);}
	| IF expression THEN expression ELSE expression
{ $$ = new IfExpression($2,$4,$6);}
	| IF expression THEN expression
{ $$ = new IfExpression($2,$4,NULL);}
	;


/*
NOTE: An else clause is associated with the most recent unmatched then clause.
*/

iteration
	: infinite_loop { $$ = $1}
	| for_loop {$$ = $1}
	| foreach_loop {$$ = $1}
	| while_loop {$$ = $1}
	| repeat_loop {$$ = $1}
	;

infinite_loop
	: LOOP expression  {$$ = new InfiniteLoop($2);}
	;


for_loop
	: FOR Symbol_Tok ASNMT expression TO expression DO expression 
{
  $$ = new ForLoop($2,$4,$6,NULL,$8);
}
	| FOR Symbol_Tok ASNMT expression TO expression BY expression DO expression 
{
  $$ = new ForLoop($2,$4,$6,$8,$10);
}
	;


foreach_loop
	: FOREACH Symbol_Tok foreach_optional_symbol optional_deeply IN expression foreach_do_or_collect expression 
{ $$ = new ForEachLoop($2,$3,$4,$6,$7,$8);}
	;

foreach_do_or_collect
	: DO { $$ = 1; }
	| COLLECT { $$ = 2;}
	;

foreach_optional_symbol
	: ',' Symbol_Tok { $$ = $2;}
	| { $$ = NULL;}
	;

optional_deeply
	: {$$ = 0;}
	| DEEPLY {$$ = 1;}
	;

while_loop
	: WHILE_TOK expression DO expression  {$$ = new WhileLoop($2,$4);}
	;

repeat_loop
	: REPEAT expression_sequence UNTIL expression {$$ = new RepeatLoop($2,$4);}
	| REPEAT expression_sequence SEMI_UNTIL expression {$$ = new RepeatLoop($2,$4);}
	;

break_expression
	: BREAK expression  { $$ = new BreakExpr($2);}
	| BREAK { $$ = new BreakExpr(NULL);}
	;

try_expression
        : TRY expression_sequence
             { $$ = new TryExpr($2);}
	| TRY expression_sequence onexception_list 
             {
               if ($3 == NULL)
                 { $$ = new TryExpr($2);}
               else
                 { $$ = $3->SetExpr($2);}
             }
	;



onexception_list
	: onexception_clause {$$ = new TryExpr($1);}
	| onexception_list onexception_clause { $$ = $1->AddClause($2)}
	;

onexception_clause
	: ONEXCEPTION Symbol_Tok DO expression  {$$ = new OnExceptionClause($2,$4);}
	| SEMI_ONEXCEPTION Symbol_Tok DO expression  {$$ = new OnExceptionClause($2,$4);}
	;

local_declaration
	: LOCAL initialization_clause_list {$$ = $2;}
	;

initialization_clause_list
	: initialization_clause { $$ = new LocalDecl($1);}
	| initialization_clause_list ',' initialization_clause {$$ = $1->AddInitClause($3);}
	;

initialization_clause
	: Symbol_Tok { $$ = new InitClause($1,NULL);}
	| Symbol_Tok ASNMT expression { $$ = new InitClause($1,$3);}
	;

constant_declaration
	: CONSTANT constant_init_clause_list { $$ = $2}
	;

constant_init_clause_list
	: constant_init_clause { $$ = new ConstantDecl($1);}
	| constant_init_clause_list ',' constant_init_clause { $$ = $1->AddConstantClause($3);}
	;

constant_init_clause
	: Symbol_Tok ASNMT expression { $$ = new ConstantInitClause($1,$3);}
	;

return_expression
	: RETURN expression { $$ = new ReturnExpr($2)}
	| RETURN   { $$ = new ReturnExpr(NULL);}
	;

global_declaration
	: GLOBAL initialization_clause { $$ = new GlobalWithInitialization($2);}
	| GLOBAL Symbol_Tok '(' optional_formal_argument_list ')' expression 
{ $$ = new GlobalFunctionDecl(1,$2,$4,$6);}
	| FUNC Symbol_Tok '(' optional_formal_argument_list ')' expression 
{ $$ = new GlobalFunctionDecl(0,$2,$4,$6);}
	;


  




