// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#include <iostream.h>
#include <DLList.h>
#include "prod.h"


virtual int Input::output(int tc)
{
  ConstituentList tempList = theConstituents;
  Constituent *tempItem;
  outtab(tc);
  cout << "Input ";
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int ExpressionSequence::output(int tc)
{
  ExpressionList tempExpressions = theExpressions;
  Expression *tempExpression;
  outtab(tc);
  cout << "ExpressionSequence  ";
  while (!tempExpressions.empty())
    {
      tempExpression = tempExpressions.remove_front();
      tempExpression->output(tc + 4);
    };
}

virtual int Symbol::output(int tc)
{
  outtab(tc);

  cout << "Symbol :" << value;
}

virtual int Symbol::dump()
{
  cout << "Symbol :" << value << " " ;
}

virtual int FormalArgumentList::output(int tc)
{
  outtab(tc);
  cout << "FormalArgumentList ";
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc + 4);
    };
}



virtual int InitClause::output(int tc)
{
  outtab(tc);
  cout << "InitClause ";
  theSymbol->output(tc+4);
  if (theExpression != NULL)
    theExpression->output(tc+4);
}

virtual int GlobalFunctionDecl::output(int tc)
{
  outtab(tc);
  cout << "GlobalFunctionDecl ";
  cout << " IsGlobal " << isGlobal;
  theSymbol->output(tc+4);
  theArgs->output(tc+4);
  theExpression->output(tc+4);
}


virtual int GlobalWithInitialization::output(int tc)
{
  outtab(tc);
  cout  << "GlobalWithInitialization";
  theInitClause->output(tc+4);
}

virtual int SimpleExpression::output(int tc)
{
  outtab(tc);
  cout << "SimpleExpression";
  
  switch(theOperator)
    {
    case 0: 
      cout << "    Operator SELF";
      break;
    case 1:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator +";
      break;
    case 2:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator -";
      break;
    case 3:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator *";
      break;
    case 4:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator /";
      break;
    case 5:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator DIV";
      break;
    case 6:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator MOD";
      break;
    case 7:	
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator LS";
      break;
    case 8:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator RS";
      break;
    case 9:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator =";
      break;
    case 10:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator NE";
      break;
    case 11:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator <";
      break;
    case 12:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator >";
      break;
    case 13:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator LE";
      break;
    case 14:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator GE";
      break;
    case 15:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator AND";
      break;
    case 16:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator OR";
      break;
    case 17:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator &";
      break;
    case 18:
      left_expr->output(tc+4);
      right_expr->output(tc+4);
      cout << "    Operator &&";
      break;
    case 19:
      left_expr->output(tc+4);
      cout << "    Operator -";
      break;
    case 20:
      left_expr->output(tc+4);
      cout << "    Operator NOT";
      break;
    case 21:
      cout << "    Operator SELF";
      break;
    }
}

virtual EValue SimpleExpression::evalExpr()
{
  EValue t;
  EValue l;
  EValue r;
  switch(theOperator)
    {
    case 0:
      break;
    case 1:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l + r;
      break;
    case 2:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l - r;
      break;
    case 3:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l * r;
      break;
    case 4:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l / r;
      break;
    case 5:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.div(r);
      break;
    case 6:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.mod(r);
      break;
    case 7:	
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.lshft(r);
      break;
    case 8:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.rshft(r);
      break;
    case 9:
      break;
    case 10:
      break;
    case 11:
      break;
    case 12:
      break;
    case 13:
      break;
    case 14:
      break;
    case 15:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.band(r);
      break;
    case 16:
      l = left_expr->evalExpr();
      r = right_expr->evalExpr();
      t = l.bor(r);
      break;
    case 17:
      break;
    case 18:
      break;
    case 19:
      l = left_expr->evalExpr();
      t = l.uminus();
      break;
    case 20:
      l = left_expr->evalExpr();
      if (l.theType == EVInt)
	t = l.not();
      break;
    case 21:
      break;
    }
  return t;
}


virtual int CompoundExpression::output(int tc)
{
  outtab(tc);
  cout << "CompoundExpression";
  if (theExpressions)
    theExpressions->output(tc+4);
}

virtual int SimpleObject::output(int tc)
{
  outtab(tc);
  cout << "SimpleObject";
  theSimpleLiteral->output(tc+4);
}

virtual int PathExpression::output(int tc)
{
  outtab(tc);
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  cout << "PathExpression ";
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}


virtual int PathObject::output(int tc)
{
  outtab(tc);
  cout << "PathObjectLiteral";
  theObject->output(tc+4);
}


virtual int Array::output(int tc)
{
  outtab(tc);
  cout << "Array ";
  if (initialSymbol != NULL)
    initialSymbol->output(tc+4);
  ObjectList tempList = theObjects;
  Object *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int ArrayObject::output(int tc)
{
  outtab(tc);
  cout << "ArrayObjectLiteral";
  theObject->output(tc+4);
}


virtual int FrameSlot::output(int tc)
{
  outtab(tc);
  cout << "FrameSlot";
  theSymbol->output(tc+4);
  theObject->output(tc+4);
}

virtual int Frame::output(int tc)
{
  outtab(tc);
  cout << "Frame ";
  FrameSlotList tempList = theSlots;
  FrameSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int FrameObject::output(int tc)
{
  outtab(tc);
  cout << "FrameObjectLiteral";
  theObject->output(tc+4);
}

virtual int ArrayConstructor::output(int tc)
{
  outtab(tc);
  cout << "ArrayConstructor ";
  if (initialSymbol != NULL)
    initialSymbol->output(tc+4);
  ExpressionList tempList = theExpressions;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}


virtual int FrameConstructorSlot::output(int tc)
{
  outtab(tc);
  cout << "FrameConstructorSlot";
  theSymbol->output(tc+4);
  theExpression->output(tc+4);
}

virtual int FrameConstructor::output(int tc)
{
  outtab(tc);
  cout << "FrameConstructor ";
  FSSlotList tempList = theSlots;
  FrameConstructorSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int FunctionConstructor::output(int tc)
{
  outtab(tc);
  cout << "FunctionConstructor";
  if (theArgs != NULL)
    theArgs->output(tc+4);
  theExpression->output(tc+4);
}


virtual int FrameAccessor::output(int tc)
{
  outtab(tc);
  cout << "FrameAccessor";
  leadingExpr->output(tc+4);
  if (theSymbol != NULL)
    theSymbol->output(tc+4);
  if (trailingExpr != NULL)
    trailingExpr->output(tc+4);
}

virtual int ArrayAccessor::output(int tc)
{
  outtab(tc);
  cout << "ArrayAccessor";
  leadingExpr->output(tc+4);
  trailingExpr->output(tc+4);
}


virtual int Assignment::output(int tc)
{
  outtab(tc);
  cout << "Assignment";
  theLvalue->output(tc+4);
  theExpression->output(tc+4);
}

virtual int SymbolExists::output(int tc)
{
  outtab(tc);
  cout << "SymbolExists";
  theSymbol->output(tc+4);
}

virtual int FrameAccExists::output(int tc)
{
  outtab(tc);
  cout << "FrameAccExists";
  theFrameAcc->output(tc+4);
}

virtual int ExprExists::output(int tc)
{
  outtab(tc);
  cout << "ExprExists";
  theExpression->output(tc+4);
  if (theSymbol)
    theSymbol->output(tc+4);
}

virtual int ActualArgumentList::output(int tc)
{
  outtab(tc);
  cout << "ActualArgumentList";
  
  ExpressionList tempList = theArgs;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}


virtual int FunctionCall::output(int tc)
{
  outtab(tc);
  cout << "FunctionCall";
  if (theSymbol != NULL)
    theSymbol->output(tc+4);
  if (theExpression != NULL)
    theExpression->output(tc+4);
  if (theArgs != NULL)
    theArgs->output(tc+4);
}

virtual int MessageSend::output(int tc)
{
  outtab(tc);
  cout << "MessageSend";
  cout << " IsInherited " << isInherited;
  if (theExpression)
    theExpression->output(tc+4);
  cout << " ColOrCQ " << col_or_cq;

  if (theSymbol)  
    theSymbol->output(tc+4);
  if (theArgs)
    theArgs->output(tc+4);
}

virtual int IfExpression::output(int tc)
{
  outtab(tc);
  cout << "IfExpression";
  if (boolExpr != NULL)
    boolExpr->output(tc+4);
  if (thenExpr != NULL)
    thenExpr->output(tc+4);
  if (elseExpr != NULL)
    elseExpr->output(tc+4);
}


virtual int InfiniteLoop::output(int tc)
{
  outtab(tc);
  cout << "InfiniteLoop ";
  theExpression->output(tc+4);
}

virtual int ForLoop::output(int tc)
{
  outtab(tc);
  cout << "ForLoop";
  loopSymbol->output(tc+4);
  initialExpression->output(tc+4);
  toExpression->output(tc+4);
  if (byExpression != NULL)
    byExpression->output(tc+4);
  doExpression->output(tc+4);
}

virtual int ForEachLoop::output(int tc)
{
  outtab(tc);
  cout << "ForEachLoop";
  initialSymbol->output(tc+4);
  if (nextSymbol != NULL)
    nextSymbol->output(tc+4);
  cout << " IsDeeply " <<isDeeply;
  inExpression->output(tc+4);
  cout << " IsDo " <<isDo;
  doExpression->output(tc+4);
}

virtual int WhileLoop::output(int tc)
{
  outtab(tc);
  cout << "WhileLoop";
  whileExpression->output(tc+4);
  doExpression->output(tc+4);
}

virtual int RepeatLoop::output(int tc)
{
  outtab(tc);
  cout << "RepeatLoop";
  theExpressions->output(tc+4);
  untilExpression->output(tc+4);
};

virtual int BreakExpr::output(int tc)
{
  outtab(tc);
  cout << "BreakExpr";
  if (theExpression != NULL)
    theExpression->output(tc+4);
}

virtual int OnExceptionClause::output(int tc)
{
  outtab(tc);
  cout << "OnExceptionClause";
  theSymbol->output(tc+4);
  theExpression->output(tc+4);
}

virtual int TryExpr::output(int tc)
{
  outtab(tc);
  cout << "TryExpr";
  theExpressions->output(tc+4);
  
  ExcClauseList tempList = theClauses;
  OnExceptionClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int LocalDecl::output(int tc)
{
  outtab(tc);
  InitClauseList tempList = theInitClauses;
  InitClause *tempItem;
  
  cout << "LocalDecl";
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int ConstantInitClause::output(int tc)
{
  outtab(tc);
  cout << "ConstantInitClause";
  theSymbol->output(tc+4);
  theExpression->output(tc+4);
};

virtual int ConstantDecl::output(int tc)
{
  outtab(tc);
  cout << "ConstantDecl ";
  
  CIClauseList tempList = theDeclClauses;
  ConstantInitClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->output(tc+4);
    };
}

virtual int ReturnExpr::output(int tc)
{
  outtab(tc);
  cout << "Return ";
  if (theExpression != NULL)
    theExpression->output(tc+4);
}


virtual int NString::output(int tc)
{
  outtab(tc);
  cout << "String " << value;
}

virtual int NInteger::output(int tc)
{
  outtab(tc);
  cout << "Integer " << value;
}

virtual int NReal::output(int tc)
{
  outtab(tc);
  cout << "real ";
}


virtual int NCharacter::output(int tc)
{
  outtab(tc);
  cout << "character" << value;
};

virtual int NTrue::output(int tc)
{
  outtab(tc);
  cout << "True";
};

virtual int NNil::output(int tc)
{
  outtab(tc);
  cout << "Nil";
}


virtual int MPointer::output(int tc)
{
  outtab(tc);
  cout << "MPointer " << value;
}



