// This may look like C code, but it is really -*- C++ -*-

/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/


#include <iostream.h>
#include "prod.h"
#include <DLList.h>
#include <String.h>
#include "symtab.h"


SymtabEntry::SymtabEntry(String nv,Symtab *t)
{
  putInFrame = 0;
  entryValue=nv;
  theTable=t;
  isMagic = 0;
  theNOSymbol = new NO_Symbol(nv);
}


EValue SymtabEntry::evalExpr()
{
  EValue t;
  if (theClass == constantSym)
    return theValue;
}


NO_VpumUnusual *GetNilSym();


SymtabEntry::AddLocalToFrame(NO_Frame *theFrame)
{
  theFrame->AddSlot(theNOSymbol,GetNilSym());  
  theObjectFile->outFile << entryValue <<endl;
}

SymtabEntry::emit(Codeblock *theCode,NO_Frame *theContainer)
{
  switch (theClass)
    {
    case otherSym:
      theObjectFile->outFile << "push otherSym" << entryValue;
      break;
    case localSym:
      theCode->AppendInstruction(BC_PUSHLOCAL,order+2,simple);
      //	  theObjectFile->outFile << "push localSym" << order;
      break;
    case globalSym:
      theCode->AppendInstruction(BC_PUSHEXTERNAL,order+2,simple);
      theObjectFile->outFile << "push globalSym" << entryValue;
      break;
    case constantSym:
      {
      switch (theValue.GetType()) 
	{
	case EVString:
	  {
	    NO_String *theString;
	    int litNum;
	    
	    theString = new NO_String(theValue.getStringValue());
	    NO_VpumPtr *tempPtr;
	    
	    tempPtr = theString->CreateVPointer();
	    litNum = theContainer->AddLiteral(tempPtr);
	    theCode->AppendInstruction(BC_PUSHLITERAL,litNum,simple);
	    break;
	  }
	case EVInt:
	  theCode->AppendInstruction(BC_PUSHIMMEDIATE,theValue.getIntValue(),simple);
	  break;
	case EVChar:
	  break;
	case EVReal:
	  break;
	case EVUlong:
	  break;
	case EVNone:
	  break;
	}
      break;
      }
    case argSym:
      theCode->AppendInstruction(BC_PUSHLOCAL,order+2,simple);
      //	  theObjectFile->outFile << "push argSym "<<order;
      break;
    default:
      theObjectFile->outFile << "????" << entryValue<<"??";
      break;
    }
}

SymtabEntry::emitAssignTo(Codeblock *theCode)
{
  switch (theClass)
    {
    case otherSym:
      theObjectFile->outFile << "push otherSym" << entryValue;
      break;
    case localSym:
      theCode->AppendInstruction(BC_ASSIGNTOLOCAL,order+2,simple);
      //	  theObjectFile->outFile << "push localSym" << order;
      break;
    case globalSym:
      theCode->AppendInstruction(BC_ASSIGNTOEXTERNAL,order+2,simple);
      theObjectFile->outFile << "push globalSym" << entryValue;
      break;
    case constantSym:
      theObjectFile->outFile << "error, cannot assign to a constant";
      break;
    case argSym:
      theCode->AppendInstruction(BC_ASSIGNTOLOCAL,order+2,simple);
      //	  theObjectFile->outFile << "push argSym "<<order;
      break;
    default:
      theObjectFile->outFile << "????" << entryValue;
      break;
    }
}

SymtabEntry::emitIncr(Codeblock *theCode)
{
  switch (theClass)
    {
    case otherSym:
      break;
    case localSym:
      theCode->AppendInstruction(BC_INCREMENTLOCAL,order+2,simple);
      break;
    case globalSym:
      break;
    case constantSym:
      theObjectFile->outFile << "error, cannot assign to a constant";
      break;
    case argSym:
      break;
    default:
      break;
    }
}

SymtabEntry::dump()
{
  cerr << entryValue << " M:" <<isMagic << " ";
  switch (theClass)
    {
    case otherSym:
      cerr << "otherSym";
      break;
    case localSym:
      cerr << "localSym " << order;
      break;
    case globalSym:
      cerr << "globalSym";
      break;
    case constantSym:
      cerr << "constantSym value ";
      theValue.dump();
      break;
    case argSym:
      cerr << "argSym " << order;
      break;
    }
}




void Symtab::PushScope(void)
{
//TODO fill in here (not needed???)
}

void Symtab::PopScope(void)
{
//TODO fill in here (not needed???)
}

SymtabEntry::SetArg(){theClass = argSym;order=theTable->NextOrder();}
SymtabEntry::SetLocal() {theClass = localSym;order=theTable->NextOrder();}


SymtabEntry *Symtab::FindEntry(String &s)
{
  String *ts;

  SymtabList tempList = theEntries;
  SymtabEntry *tempItem;
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      ts = tempItem->GetValue();
      if (*ts == s)
	return tempItem;
    }
  // look in parent table
  if (parent != NULL)
    return parent->FindEntry(s);
  return NULL;
}



SymtabEntry *Symtab::LookupEntry(String &s)
{
  const char *ss = (const char *)s;
  SymtabEntry *tempSymb;


  tempSymb = FindEntry(s);
  if (tempSymb == NULL)
    {
      tempSymb = new SymtabEntry(ss,this);
      theEntries.append(tempSymb);
    }
  return tempSymb;
}


void Symtab::dump(char *hd)
{
  SymtabList tempList = theEntries;
  SymtabEntry *tempItem;
  cerr << endl << "-----" << hd << " Symbol Table----------" << endl;
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->dump();
      cerr << endl;
    }
  cerr << "---------------------------" << endl;
}


NO_Symbol *Symtab::getNOSymbol(String &s)
{
  SymtabEntry *tempSymb;
  NO_Symbol *nos;
  tempSymb = LookupEntry(s);
  nos = tempSymb->getNOSymbol();
  return nos;
}
