// This may look like C code, but it is really -*- C++ -*-
/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/
#ifndef SYMTAB_H_INC
#define SYMTAB_H_INC
#include <String.h>
#include "evalue.h"
#include "newtobjs.h"

extern ObjectFile *theObjectFile;
class Symtab;

enum SymbolClass 
{otherSym,localSym,globalSym,constantSym,argSym};

class SymtabEntry
{
  enum SymbolClass theClass;
  String entryValue;
  int order;
  Symtab *theTable;
  int isMagic;
  int putInFrame;
  NO_Symbol *theNOSymbol;
 public:
  EValue theValue;

  SymtabEntry(String nv,Symtab *t);

  dump();
  emitIncr(Codeblock *theCode);
  emitAssignTo(Codeblock *theCode);
  emit(Codeblock *theCode,NO_Frame *theContainer);
  SetLocal();
  SetArg();
  MarkMagic() {isMagic = 1;}
  SetGlobal(){theClass = globalSym;}
  SetConstant(){theClass = constantSym;}
  SetOther(){theClass = otherSym;}
  Order(){return order;}
  int isLocal() {return theClass == localSym;}
  isConstant() {return (theClass == constantSym);}
  PutInFrame() {putInFrame = 1;}
  AddLocalToFrame(NO_Frame *theFrame);
  EValue evalExpr();
  String *GetValue() {return &entryValue;}
  NO_Symbol *getNOSymbol() {return theNOSymbol;}
      
};
typedef DLList<SymtabEntry *> SymtabList;


class Symtab
{
  SymtabList theEntries;
  Symtab *parent;
  int thisArgOrLocal;
  
 public:
  Symtab() {thisArgOrLocal = 0;parent = NULL;}
  Symtab(Symtab *pp) {thisArgOrLocal = 0;parent = pp;}
  SymtabEntry *LookupEntry(String &s);
  SymtabEntry *FindEntry(String &s);
  SymtabEntry *FindEntry(char *sp)
    {
      String ts(sp);
      return FindEntry(ts);
    }
  SetParent(Symtab *pp) {parent = pp;}
  void dump(char * hd);
  void PushScope(void);
  void PopScope(void);
  NO_Symbol *getNOSymbol(String &s);
  int NextOrder()
    {
      return ++thisArgOrLocal;
    }
  void AddLocalsToFrame(NO_Frame *argFrame)
    {
      SymtabList tempList = theEntries;
      SymtabEntry *tempItem;
      while (!tempList.empty())
	{
	  tempItem = tempList.remove_front();
	  if (tempItem->isLocal())
	    tempItem->AddLocalToFrame(argFrame);
	}

    }
};


#endif
