// This may look like C code, but it is really -*- C++ -*-

/*
    newtonsc NewtonScript compiler
    Copyright (C) 1995  Ross Sponholtz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



*/

#include <iostream.h>
#include <DLList.h>
#include "prod.h"


virtual int Input::walk(Symtab *context)
{
  ConstituentList tempList = theConstituents;
  Constituent *tempItem;

  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}

virtual int ExpressionSequence::walk(Symtab *context)
{
  ExpressionList tempExpressions = theExpressions;
  Expression *tempExpression;

  while (!tempExpressions.empty())
    {
      tempExpression = tempExpressions.remove_front();
      tempExpression->walk(context);
    };
}

virtual int Symbol::walk(Symtab *context)
{
  //look for the same symbol in the symbol table
  symbolTableRef = context->LookupEntry(value);
}

virtual int Symbol::MarkMagic()
{
  //look for the same symbol in the symbol table
  symbolTableRef->MarkMagic();
}

virtual int FormalArgumentList::walk(Symtab *context)
{
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
      tempItem->SetArg();
    };
}



virtual int InitClause::walk(Symtab *context)
{
  theSymbol->walk(context);
  if (theExpression != NULL)
    theExpression->walk(context);
}

virtual int GlobalFunctionDecl::walk(Symtab *context)
{
  theSymbol->walk(context);
  theArgs->walk(context);
  theExpression->walk(context);
}


virtual int GlobalWithInitialization::walk(Symtab *context)
{
  theInitClause->walk(context);
  theInitClause->MarkGlobal();
}

virtual int SimpleExpression::walk(Symtab *context)
{
  switch(theOperator)
    {
    case 0: 
      break;
    case 1:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 2:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 3:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 4:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 5:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 6:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 7:	
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 8:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 9:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 10:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 11:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 12:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 13:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 14:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 15:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 16:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 17:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 18:
      left_expr->walk(context);
      right_expr->walk(context);
      break;
    case 19:
      left_expr->walk(context);
      break;
    case 20:
      left_expr->walk(context);
      break;
    case 21:
      break;
    }
}


virtual int CompoundExpression::walk(Symtab *context)
{
  if (theExpressions)
    theExpressions->walk(context);
}

virtual int SimpleObject::walk(Symtab *context)
{
  theSimpleLiteral->walk(context);
}

virtual int PathExpression::walk(Symtab *context)
{
  SymbolList tempList = theSymbols;
  Symbol *tempItem;
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}


virtual int PathObject::walk(Symtab *context)
{
  theObject->walk(context);
}


virtual int Array::walk(Symtab *context)
{
  if (initialSymbol != NULL)
    initialSymbol->walk(context);
  ObjectList tempList = theObjects;
  Object *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}

virtual int ArrayObject::walk(Symtab *context)
{
  theObject->walk(context);
}


virtual int FrameSlot::walk(Symtab *context)
{
  theSymbol->walk(context);
  theObject->walk(context);
}

virtual int Frame::walk(Symtab *context)
{
  FrameSlotList tempList = theSlots;
  FrameSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}

virtual int FrameObject::walk(Symtab *context)
{
  theObject->walk(context);
}

virtual int ArrayConstructor::walk(Symtab *context)
{
  if (initialSymbol != NULL)
    initialSymbol->walk(context);
  ExpressionList tempList = theExpressions;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}


virtual int FrameConstructorSlot::walk(Symtab *context)
{
  theSymbol->walk(context);
  theExpression->walk(context);
}

virtual int FrameConstructor::walk(Symtab *context)
{
  FSSlotList tempList = theSlots;
  FrameConstructorSlot *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}

virtual int FunctionConstructor::walk(Symtab *context)
{
  //create a new local symbol table
  if (theArgs != NULL)
    theArgs->walk(&theSymbolTable);
  theExpression->walk(&theSymbolTable);
}


virtual int FrameAccessor::walk(Symtab *context)
{
  leadingExpr->walk(context);
  if (theSymbol != NULL)
    theSymbol->walk(context);
  if (trailingExpr != NULL)
    trailingExpr->walk(context);
}

virtual int ArrayAccessor::walk(Symtab *context)
{
  leadingExpr->walk(context);
  trailingExpr->walk(context);
}


virtual int Assignment::walk(Symtab *context)
{
  theLvalue->walk(context);
  theExpression->walk(context);
}

virtual int SymbolExists::walk(Symtab *context)
{
  theSymbol->walk(context);
}

virtual int FrameAccExists::walk(Symtab *context)
{
  theFrameAcc->walk(context);
}

virtual int ExprExists::walk(Symtab *context)
{
  if (theExpression)
    theExpression->walk(context);
  theSymbol->walk(context);
}

virtual int ActualArgumentList::walk(Symtab *context)
{
  
  ExpressionList tempList = theArgs;
  Expression *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}


virtual int FunctionCall::walk(Symtab *context)
{
  if (theSymbol != NULL)
    theSymbol->walk(context);
  if (theExpression != NULL)
    theExpression->walk(context);
  if (theArgs != NULL)
    theArgs->walk(context);
}

virtual int MessageSend::walk(Symtab *context)
{
  if (theExpression)
    theExpression->walk(context);
  if (theSymbol)
    theSymbol->walk(context);
  if (theArgs)
    theArgs->walk(context);
}

virtual int IfExpression::walk(Symtab *context)
{
  if (boolExpr != NULL)
    boolExpr->walk(context);
  if (thenExpr != NULL)
    thenExpr->walk(context);
  if (elseExpr != NULL)
    elseExpr->walk(context);
}


virtual int InfiniteLoop::walk(Symtab *context)
{
  theExpression->walk(context);
}


virtual int ForLoop::walk(Symtab *context)
{
  Symtab *st;
  SymtabEntry *limitEnt;
  SymtabEntry *incrEnt;

  String symb = loopSymbol->SymbolValue();

  limitSymbol = new Symbol(symb + "|limit");
  incrSymbol = new Symbol(symb + "|incr");

  loopSymbol->walk(context);
  loopSymbol->SetLocal();
  limitSymbol->walk(context);
  limitSymbol->SetLocal();
  incrSymbol->walk(context);
  incrSymbol->SetLocal();


  initialExpression->walk(context);
  toExpression->walk(context);
  if (byExpression != NULL)
    byExpression->walk(context);
  doExpression->walk(context);
}

virtual int ForEachLoop::walk(Symtab *context)
{
  initialSymbol->walk(context);
  if (nextSymbol != NULL)
    nextSymbol->walk(context);
  inExpression->walk(context);
  doExpression->walk(context);
}

virtual int WhileLoop::walk(Symtab *context)
{
  whileExpression->walk(context);
  doExpression->walk(context);
}

virtual int RepeatLoop::walk(Symtab *context)
{
  theExpressions->walk(context);
  untilExpression->walk(context);
};

virtual int BreakExpr::walk(Symtab *context)
{
  if (theExpression != NULL)
    theExpression->walk(context);
}

virtual int OnExceptionClause::walk(Symtab *context)
{
  theSymbol->walk(context);
  theExpression->walk(context);
}

virtual int TryExpr::walk(Symtab *context)
{
  theExpressions->walk(context);
  
  ExcClauseList tempList = theClauses;
  OnExceptionClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
    };
}

virtual int LocalDecl::walk(Symtab *context)
{
  InitClauseList tempList = theInitClauses;
  InitClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      tempItem->walk(context);
      tempItem->MarkLocal();
    };
}

virtual int ConstantInitClause::walk(Symtab *context)
{
  EValue tt;
  SymtabEntry *theRef;
  theSymbol->walk(context);
  theSymbol->SetConstant();
  if (isMagic) 
    theSymbol->MarkMagic();
  theExpression->walk(context);
  theRef = theSymbol->GetRef();
  tt = theExpression->evalExpr();
  theRef->theValue = tt;
  theRef->SetConstant();
};

virtual int ConstantDecl::walk(Symtab *context)
{
  
  CIClauseList tempList = theDeclClauses;
  ConstantInitClause *tempItem;
  
  while (!tempList.empty())
    {
      tempItem = tempList.remove_front();
      
      tempItem->walk(context);
      tempItem->MarkConstant();
      if (isMagic)
	tempItem->MarkMagic();
    };
}

virtual int ReturnExpr::walk(Symtab *context)
{
  if (theExpression != NULL)
    theExpression->walk(context);
}


virtual int NString::walk(Symtab *context)
{
}

virtual int NInteger::walk(Symtab *context)
{
}

virtual int NReal::walk(Symtab *context)
{
}

virtual int NCharacter::walk(Symtab *context)
{
};

virtual int NTrue::walk(Symtab *context)
{
};

virtual int NNil::walk(Symtab *context)
{
}

virtual int MPointer::walk(Symtab *context)
{
}




